@@ -1,12 +1,14 @@
 #undef ENABLE_NLS
 #undef HAVE_CATGETS
 #undef HAVE_GETTEXT
+#undef GETTEXT_PACKAGE
 #undef HAVE_LC_MESSAGES
 #undef HAVE_STPCPY
 #undef HAVE_LIBSM
 #undef PACKAGE_LOCALE_DIR
 #undef PACKAGE_DATA_DIR
 #undef PACKAGE_SOURCE_DIR
+#undef NAUTILUS_PREFIX
 #undef CVS_SOURCE 
 #undef CORBA_INTERFACE
 #undef MOZILLA_VERSION
@@ -19,4 +21,8 @@
 #undef XPCOM_SHARED_LIB
 #undef ENABLE_NAUTILUS_VIEW
 #undef STRING_HEADER
-#undef GNOME_PIXMAPS_DIR
+#undef ENABLE_MOZILLA_EMBED
+#undef ENABLE_GTKHTML_EMBED
+#undef ENABLE_JUST_GNOMEVFS
+#undef ENABLE_SIMPLE_MENUBAR
+#undef MAINTAINER_MODE
