@@ -84,8 +84,7 @@ static void		gb_editor_show_data		(GbEditor *e);
 static void		gb_editor_add_new		(GbEditor *e, GbBookmark *new_bookmark);
 static void 		gb_editor_add_several		(GbEditor *e, GSList *l);
 static void		gb_editor_bookmark_activated_cb	(GObject *sender,
-							 GbBookmark *b, const gchar *url, 
-							 GbBookmarkActivatedFlags f, 
+							 GbBookmarkEventActivated *ev,
 							 GbEditor *e);
 static gboolean		gb_editor_window_configure_event_cb (GtkWidget *widget,
 							     GdkEventConfigure *event,
@@ -94,6 +93,8 @@ static gboolean		gb_editor_window_configure_event_cb (GtkWidget *widget,
 
 static void		gb_editor_bonobo_cmd_add_site		(BonoboUIComponent *uic, GbEditor *e,
 								 const gchar* verbname);
+static void		gb_editor_bonobo_cmd_add_smart_site	(BonoboUIComponent *uic, GbEditor *e,
+								 const gchar* verbname);
 static void		gb_editor_bonobo_cmd_add_folder		(BonoboUIComponent *uic, GbEditor *e,
 								 const gchar* verbname);
 static void		gb_editor_bonobo_cmd_add_separator	(BonoboUIComponent *uic, GbEditor *e,
@@ -147,6 +148,7 @@ static void		gb_editor_bonobo_cmd_edit_goto		(BonoboUIComponent *uic, GbEditor *
 
 BonoboUIVerb bookmarks_editor_verbs [] = {
         BONOBO_UI_VERB ("AddSite", (BonoboUIVerbFn)gb_editor_bonobo_cmd_add_site),
+        BONOBO_UI_VERB ("AddSmartSite", (BonoboUIVerbFn)gb_editor_bonobo_cmd_add_smart_site),
 	BONOBO_UI_VERB ("AddFolder", (BonoboUIVerbFn)gb_editor_bonobo_cmd_add_folder),
 	BONOBO_UI_VERB ("AddSeparator", (BonoboUIVerbFn) gb_editor_bonobo_cmd_add_separator),
         BONOBO_UI_VERB ("AddAlias", (BonoboUIVerbFn) gb_editor_bonobo_cmd_add_alias),
@@ -222,8 +224,8 @@ gb_editor_class_init (GbEditorClass *klass)
 		G_SIGNAL_RUN_FIRST | G_SIGNAL_RUN_LAST | G_SIGNAL_RUN_CLEANUP,
                 G_STRUCT_OFFSET (GbEditorClass, gb_editor_bookmark_activated), 
 		NULL, NULL, 
-		galeon_marshal_VOID__OBJECT_STRING_INT,
-		G_TYPE_NONE, 3, GB_TYPE_BOOKMARK, G_TYPE_STRING, G_TYPE_UINT);
+		g_cclosure_marshal_VOID__POINTER,
+		G_TYPE_NONE, 1, G_TYPE_POINTER);
 	
 	g_object_class = g_type_class_peek_parent (klass);
 }
@@ -502,11 +504,10 @@ gb_editor_window_configure_event_cb (GtkWidget *widget,
 
 static void
 gb_editor_bookmark_activated_cb (GObject *sender,
-				 GbBookmark *b, const gchar *url, 
-				 GbBookmarkActivatedFlags f, 
+				 GbBookmarkEventActivated *ev,
 				 GbEditor *e)
 {
-	g_signal_emit_by_name (e, "bookmark-activated", b, url, f);
+	g_signal_emit_by_name (e, "bookmark-activated", ev);
 }
 
 static void 
@@ -586,8 +587,25 @@ gb_editor_bonobo_cmd_add_site (BonoboUIComponent *uic, GbEditor *e, const gchar*
 void 
 gb_editor_cmd_add_site (GbEditor *e)
 {
+	/* see also bookmarks-context-menu.c */
 	GbSite *new = gb_site_new (gb_tree_model_get_set (e->priv->btree_model),
-				   _("New Site"), _("http://"));
+				   _("New Site"), "http://");
+	gb_editor_add_new (e, GB_BOOKMARK (new));
+	g_object_unref (G_OBJECT (new));
+}
+
+static void
+gb_editor_bonobo_cmd_add_smart_site (BonoboUIComponent *uic, GbEditor *e, const gchar* verbname)
+{
+	gb_editor_cmd_add_smart_site (e);
+}
+
+void 
+gb_editor_cmd_add_smart_site (GbEditor *e)
+{
+	/* see also bookmarks-context-menu.c */
+	GbSmartSite *new = gb_smart_site_new (gb_tree_model_get_set (e->priv->btree_model),
+					      _("New Smart Site"), "http://", "http://%s");
 	gb_editor_add_new (e, GB_BOOKMARK (new));
 	g_object_unref (G_OBJECT (new));
 }
@@ -950,13 +968,13 @@ gb_editor_cmd_edit_goto (GbEditor *e)
 	{
 		GbBookmark *b = selected->data;
 		const gchar *url = GB_IS_SITE (b) ? GB_SITE (b)->url : NULL;
-		g_signal_emit_by_name (e, "bookmark-activated", b, url, GB_BAF_DEFAULT);
+		gb_activated_activate (e, b, url, GB_BAF_DEFAULT);
 	}
 	else while (selected)
 	{
 		GbBookmark *b = selected->data;
 		const gchar *url = GB_IS_SITE (b) ? GB_SITE (b)->url : NULL;
-		g_signal_emit_by_name (e, "bookmark-activated", b, url, GB_BAF_NEW_TAB_OR_WINDOW);
+		gb_activated_activate (e, b, url, GB_BAF_NEW_TAB_OR_WINDOW);
 		selected = selected->next;
 	}
 }
