@@ -17,6 +17,7 @@
  */
 
 #include "downloader-view.h"
+#include "disclosure-widget.h"
 #include "eel-gconf-extensions.h"
 #include "gul-gui.h"
 #include "prefs-strings.h"
@@ -33,6 +34,13 @@
 #include <gtk/gtkprogressbar.h>
 #include <libgnomevfs/gnome-vfs-mime-handlers.h>
 
+/* WHACK WHACK WHACK FIXME please
+ * (nevermind the type mismatch, we'd typecast it away anyway)
+ * This works, but isn't really nice.  We shouldn't be referencing outside
+ * embed/ directory, but I just can't seem to find a better solution...
+ */
+extern GObject *galeon_shell;
+
 enum
 {
 	DOWNLOAD_REMOVE,
@@ -279,6 +287,10 @@ downloader_view_init (DownloaderView *dv)
 							(GDestroyNotify)destroy_details_cb);
 	
 	downloader_view_build_ui (dv);
+
+	/* make sure galeon doesn't exit if the user closes the last window
+	 * while we're still downloading stuff */
+	g_object_ref (galeon_shell);
 }
 
 static void
@@ -298,6 +310,9 @@ downloader_view_finalize (GObject *object)
 	g_free (dv->priv);
 
         G_OBJECT_CLASS (parent_class)->finalize (object);
+
+	/* we're done, you may go away now */
+	g_object_unref (galeon_shell);
 }
 
 DownloaderView *
@@ -423,18 +438,20 @@ downloader_view_update_details (DownloaderViewPrivate *priv,
 static gboolean
 get_selected_row (DownloaderViewPrivate *priv, GtkTreeIter *iter)
 {
-	GList *l;
+	GList *refs;
 	GtkTreeRowReference *node;
+	GtkTreePath *path;
 	
-	l = gul_gui_treeview_get_selection_refs 
+	refs = gul_gui_treeview_get_selection_refs 
 		(GTK_TREE_VIEW(priv->treeview));
-	if (l == NULL) return FALSE;
+	if (refs == NULL) return FALSE;
 	
-	node = l->data;
-	gtk_tree_model_get_iter (GTK_TREE_MODEL (priv->model), iter,
-                                 gtk_tree_row_reference_get_path (node));
+	node = refs->data;
+	path = gtk_tree_row_reference_get_path (node);
+	gtk_tree_model_get_iter (GTK_TREE_MODEL (priv->model), iter, path);
+	gtk_tree_path_free (path);
 
-	g_list_free (l);
+	gul_gui_treeview_free_selection_refs (refs);
 
 	return TRUE;
 }
@@ -515,6 +532,8 @@ downloader_view_set_download_info (DownloaderViewPrivate *priv,
 	{
 		downloader_view_update_details (priv, details);
 	}
+	gtk_tree_path_free (selected_path);
+	gtk_tree_path_free (path);
 }
 
 static void
@@ -547,6 +566,7 @@ downloader_view_add_download (DownloaderView *dv,
 			      gpointer persist_object)
 {
 	GtkTreeIter iter;
+	GtkTreePath *path;
 	DownloadDetails *details;
 	GtkTreeSelection *selection;
 	
@@ -569,10 +589,10 @@ downloader_view_add_download (DownloaderView *dv,
 	gtk_list_store_append (GTK_LIST_STORE (dv->priv->model), 
 			       &iter);
 
+	path = gtk_tree_model_get_path (GTK_TREE_MODEL(dv->priv->model), &iter);
 	details->ref = gtk_tree_row_reference_new
-		(GTK_TREE_MODEL (dv->priv->model),
-		 gtk_tree_model_get_path
-		 (GTK_TREE_MODEL (dv->priv->model), &iter));
+		(GTK_TREE_MODEL (dv->priv->model), path);
+	gtk_tree_path_free (path);
 
 	gtk_list_store_set (GTK_LIST_STORE (dv->priv->model),
 			    &iter,
@@ -596,14 +616,16 @@ downloader_view_remove_download (DownloaderView *dv,
 	DownloadDetails *details;
 	GtkTreeIter iter;
 	GValue keep_open = {0, };
+	GtkTreePath *path;
 
 	details = g_hash_table_lookup (dv->priv->details_hash,
 				       persist_object);
 	g_return_if_fail (details);
 
+	path = gtk_tree_row_reference_get_path (details->ref);
 	gtk_tree_model_get_iter (GTK_TREE_MODEL (dv->priv->model), 
-				 &iter,
-				 gtk_tree_row_reference_get_path (details->ref));
+				 &iter, path);
+	gtk_tree_path_free (path);
 
 	gtk_list_store_remove (GTK_LIST_STORE (dv->priv->model), &iter);
 
@@ -635,6 +657,7 @@ downloader_view_set_download_progress (DownloaderView *dv,
 				       gpointer persist_object)
 {
 	DownloadDetails *details;
+	GtkTreePath *path;
 	GtkTreeIter iter;
 
 	details = g_hash_table_lookup (dv->priv->details_hash,
@@ -649,9 +672,10 @@ downloader_view_set_download_progress (DownloaderView *dv,
 	details->can_pause = can_pause;
 	details->progress = progress;
 
+	path = gtk_tree_row_reference_get_path (details->ref);
 	gtk_tree_model_get_iter (GTK_TREE_MODEL (dv->priv->model), 
-				 &iter,
-				 gtk_tree_row_reference_get_path (details->ref));
+				 &iter, path);
+	gtk_tree_path_free (path);
 
 	downloader_view_set_download_info (dv->priv, details, &iter); 
 }
@@ -662,6 +686,7 @@ downloader_view_set_download_status (DownloaderView *dv,
 				     gpointer persist_object)
 {
 	DownloadDetails *details;
+	GtkTreePath *path;
 	GtkTreeIter iter;
 	GValue keep_open = {0, };
 
@@ -671,9 +696,10 @@ downloader_view_set_download_status (DownloaderView *dv,
 
 	details->status = status;
 
+	path = gtk_tree_row_reference_get_path (details->ref);
 	gtk_tree_model_get_iter (GTK_TREE_MODEL (dv->priv->model), 
-				 &iter,
-				 gtk_tree_row_reference_get_path (details->ref));
+				 &iter, path);
+	gtk_tree_path_free (path);
 
 	downloader_view_set_download_info (dv->priv, details, &iter);
 	downloader_view_update_controls (dv->priv);
@@ -717,7 +743,15 @@ downloader_view_build_ui (DownloaderView *dv)
 	priv->open_button = galeon_dialog_get_control (d, PROP_OPEN_BUTTON);
 	details_table = galeon_dialog_get_control (d, PROP_DETAILS_TABLE);
 
+	/* Set inital visibility state of details */
+	download_dialog_details_cb(GTK_TOGGLE_BUTTON(priv->details_button), dv);
+
 	/* create file and location details labels */
+	cddb_disclosure_set_labels (CDDB_DISCLOSURE (priv->details_button),
+				    _("Download Details"),
+				    _("Download Details"));
+
+
 	priv->details_location = gul_ellipsizing_label_new ("");
 	priv->details_file = gul_ellipsizing_label_new ("");
 	gtk_table_attach_defaults (GTK_TABLE(details_table), priv->details_location,
@@ -869,20 +903,23 @@ download_dialog_pause_cb (GtkButton *button, DownloaderView *dv)
 void 
 download_dialog_abort_cb (GtkButton *button, DownloaderView *dv)
 {
-	GList *l;
+	GList *refs, *l;
 	
-	l = gul_gui_treeview_get_selection_refs (GTK_TREE_VIEW(dv->priv->treeview));
+	refs = gul_gui_treeview_get_selection_refs (GTK_TREE_VIEW(dv->priv->treeview));
 
-        for (l = g_list_first(l); l != NULL; l = g_list_next (l))
+        for (l = refs; l != NULL; l = l->next)
         {
                 GtkTreeRowReference *node = l->data;
 		GValue val = {0, };
 		gpointer *persist_object;
 		GtkTreeIter iter;
+		GtkTreePath *path;
 		DownloadDetails *details;
 			
+		path = gtk_tree_row_reference_get_path (node);
                 gtk_tree_model_get_iter (GTK_TREE_MODEL (dv->priv->model), &iter,
-                                         gtk_tree_row_reference_get_path (node));
+					 path);
+		gtk_tree_path_free (path);
 	
 		gtk_tree_model_get_value (dv->priv->model, &iter, 
 					  COL_PERSIST_OBJECT, &val);
@@ -895,11 +932,9 @@ download_dialog_abort_cb (GtkButton *button, DownloaderView *dv)
 		g_signal_emit (G_OBJECT (dv), downloader_view_signals[DOWNLOAD_REMOVE], 0);
 		
 		downloader_view_remove_download (dv, persist_object);
-		
-                gtk_tree_row_reference_free (node);
         }
 
-        g_list_free (l);
+	gul_gui_treeview_free_selection_refs (refs);
 }
 
 static void
@@ -1006,27 +1041,35 @@ download_dialog_delete_cb (GtkWidget *window, GdkEventAny *event,
 }
 
 void
-download_dialog_details_cb (GtkToggleButton *button, 
-			    DownloaderView *dv)
+download_dialog_details_cb(GtkToggleButton *button, 
+			   DownloaderView *dv)
 {
 	GtkWidget *details_frame;
 	GtkWidget *details_separator;
+	GtkWidget *dialog;
 
-	details_frame = galeon_dialog_get_control (GALEON_DIALOG(dv), 
-						   PROP_DETAILS_FRAME);
-	details_separator = galeon_dialog_get_control (GALEON_DIALOG(dv), 
-						       PROP_DETAILS_SEPARATOR);
-	
-	if (gtk_toggle_button_get_active (button))
+	details_frame = galeon_dialog_get_control(GALEON_DIALOG(dv), 
+						  PROP_DETAILS_FRAME);
+	details_separator = galeon_dialog_get_control(GALEON_DIALOG(dv), 
+						      PROP_DETAILS_SEPARATOR);
+
+	dialog = gtk_widget_get_toplevel(GTK_WIDGET(button));
+
+
+	if (gtk_toggle_button_get_active(button))
 	{
-		gtk_widget_show (GTK_WIDGET (details_frame));
-		gtk_widget_show (GTK_WIDGET (details_separator));
+		gtk_widget_show(GTK_WIDGET(details_frame));
+		gtk_widget_show(GTK_WIDGET(details_separator));
+		gtk_window_set_resizable(GTK_WINDOW(dialog),
+					 TRUE);
 		dv->priv->show_details = TRUE;
 	}
 	else
 	{
-		gtk_widget_hide (GTK_WIDGET (details_frame));
-		gtk_widget_hide (GTK_WIDGET (details_separator));
+		gtk_widget_hide(GTK_WIDGET(details_frame));
+		gtk_widget_hide(GTK_WIDGET(details_separator));
+		gtk_window_set_resizable(GTK_WINDOW(dialog),
+					 FALSE);
 		dv->priv->show_details = FALSE;
 	}
 
