@@ -20,6 +20,7 @@
 #include "eel-gconf-extensions.h"
 #include "gul-string.h"
 #include "gul-general.h"
+#include "gul-gobject-misc.h"
 #include "galeon-config.h"
 #include "galeon-autocompletion-source.h"
 
@@ -111,46 +112,9 @@ static GObjectClass *parent_class = NULL;
 
 static guint global_history_signals[LAST_SIGNAL] = { 0 };
 
-GType 
-global_history_get_type (void)
-{
-        static GType global_history_type = 0;
-
-        if (global_history_type == 0)
-        {
-                static const GTypeInfo our_info =
-                {
-                        sizeof (GlobalHistoryClass),
-                        NULL, /* base_init */
-                        NULL, /* base_finalize */
-                        (GClassInitFunc) global_history_class_init,
-                        NULL,
-                        NULL, /* class_data */
-                        sizeof (GlobalHistory),
-                        0, /* n_preallocs */
-                        (GInstanceInitFunc) global_history_init
-                };
-		
-		static const GInterfaceInfo autocompletion_source_info =
-			{
-				(GInterfaceInitFunc) global_history_autocompletion_source_init,
-				NULL,
-				NULL
-			};
-
-                global_history_type = g_type_register_static (G_TYPE_OBJECT,
-							      "GlobalHistory",
-							      &our_info, 0);
-
-		g_type_add_interface_static (global_history_type,
-					     GALEON_TYPE_AUTOCOMPLETION_SOURCE,
-					     &autocompletion_source_info);
-
-        }
-
-        return global_history_type;
-
-}
+MAKE_GET_TYPE_IFACE (global_history, "GlobalHistory", GlobalHistory,
+		     global_history_class_init, global_history_init, G_TYPE_OBJECT,
+		     global_history_autocompletion_source_init, GALEON_TYPE_AUTOCOMPLETION_SOURCE);
 
 static void
 global_history_autocompletion_source_init (GaleonAutocompletionSourceIface *iface)
