@@ -32,8 +32,6 @@
 #include "nsXPIDLString.h"
 #include "nsIChannel.h"
 #include "nsIFTPChannel.h"
-#include "nsIFactory.h"
-#include "nsXPComFactory.h"
 #include "nsIMIMEInfo.h"
 #include "nsCOMPtr.h"
 
@@ -54,6 +52,8 @@ NS_IMPL_ISUPPORTS4 (GProgressListener, nsIDownload, nsIWebProgressListener,
 GProgressListener::GProgressListener () : mLauncher(nsnull),
 					  mPersist(nsnull),
 				          mHandler(nsnull),
+					  mGaleonPersist(nsnull),
+					  mDownloaderView(nsnull),
 					  mObserver(nsnull),
 					  mMIMEInfo(nsnull),
 					  mPercentComplete(0)
@@ -63,7 +63,15 @@ GProgressListener::GProgressListener () : mLauncher(nsnull),
 
 GProgressListener::~GProgressListener ()
 {
-	/* destructor code */
+	if (mDownloaderView)
+	{
+		g_object_remove_weak_pointer (G_OBJECT (mDownloaderView), (void **) &mDownloaderView);
+	}
+	if (mGaleonPersist)
+	{
+		g_object_unref (mGaleonPersist);
+		mGaleonPersist = NULL;
+	}
 }
 
 NS_METHOD GProgressListener::InitForPersist (nsIWebBrowserPersist *aPersist,
@@ -85,7 +93,7 @@ NS_METHOD GProgressListener::InitForPersist (nsIWebBrowserPersist *aPersist,
 	mFile = aFile;
 	mPersist = aPersist;
 	mTimeDownloadStarted = aTimeDownloadStarted;
-	mGaleonPersist = galeonPersist;
+	mGaleonPersist = (GaleonEmbedPersist *) g_object_ref (galeonPersist);
 
 	/* do remaining init */
 	rv = PrivateInit ();
@@ -104,8 +112,6 @@ NS_METHOD GProgressListener::InitForDownload (nsIHelperAppLauncher *aLauncher,
 {
 	nsresult rv;
 
-	mNoDialog = 0;
-
 	/* fill in download details */
 	mAction = aAction;
 	mParent = do_QueryInterface (aContext);
@@ -178,6 +184,7 @@ NS_METHOD GProgressListener::PrivateInit (void)
 				  G_CALLBACK (download_resume_cb),
 				  this);
 		mDownloaderView = dv;
+		g_object_add_weak_pointer (G_OBJECT (mDownloaderView), (void **) &mDownloaderView);
 	}
 
 	/* done */
@@ -385,10 +392,12 @@ NS_IMETHODIMP GProgressListener::OnStateChange (nsIWebProgress *aWebProgress,
 				}
 
 				g_signal_emit_by_name (mGaleonPersist, "completed");
+				g_object_unref (mGaleonPersist);
+				mGaleonPersist = NULL;
 			}
 		}
 		
-		if (!mNoDialog)
+		if (!mNoDialog && mDownloaderView)
 		{
 			downloader_view_set_download_status (mDownloaderView, 
 							     DOWNLOAD_STATUS_COMPLETED, 
@@ -510,15 +519,18 @@ NS_IMETHODIMP GProgressListener::
 				/currentRate +.5);
 	}
 
-	downloader_view_set_download_progress (mDownloaderView,
-					       mElapsed,
-					       remaining,
-					       speed,
-					       totalKBytes,
-					       currentKBytes,
-					       progress,
-					       mCanPause,
-					       (gpointer)this);
+	if (mDownloaderView)
+	{
+		downloader_view_set_download_progress (mDownloaderView,
+						       mElapsed,
+						       remaining,
+						       speed,
+						       totalKBytes,
+						       currentKBytes,
+						       progress,
+						       mCanPause,
+						       (gpointer)this);
+	}
 
 	/* done */
 	return NS_OK;
@@ -661,6 +673,12 @@ nsresult GProgressListener::Abort (void)
                               nsIWebProgressListener::STATE_STOP, 0);
         }
 
+	if (mGaleonPersist)
+	{
+		g_object_unref (mGaleonPersist);
+		mGaleonPersist = NULL;
+	}
+
         if (mPersist)
         {
                 return mPersist->CancelSave ();
@@ -676,28 +694,6 @@ nsresult GProgressListener::Abort (void)
         }
 }
 
-NS_DEF_FACTORY (GProgressListener, GProgressListener);
-
-/**
- * NS_NewProgressListenerFactory:
- */ 
-nsresult NS_NewProgressListenerFactory(nsIFactory** aFactory)
-{
-        NS_ENSURE_ARG_POINTER(aFactory);
-        *aFactory = nsnull;
-
-        nsGProgressListenerFactory *result = new nsGProgressListenerFactory;
-        if (result == NULL)
-        {
-                return NS_ERROR_OUT_OF_MEMORY;
-        }
-    
-        NS_ADDREF(result);
-        *aFactory = result;
-
-        return NS_OK;
-}
-
 static void
 download_remove_cb (DownloaderView *dv, GProgressListener *Progress)
 {
