@@ -21,14 +21,11 @@
 #include "pixbuf-cache.h"
 
 #include "nsCOMPtr.h"
-#include "nsIFactory.h"
 #include "nsString.h"
 #include "nsReadableUtils.h"
 #include "nsIServiceManager.h"
-#include "nsXPComFactory.h"
 #include "MozillaPrivate.h"
 
-#include "nsIPromptService.h"
 #include "nsIUnicodeEncoder.h"
 
 #include <gtk/gtkentry.h>
@@ -50,25 +47,6 @@ static void set_editable (GtkWidget *entry, PRUnichar **text);
 static void get_check_button (GtkWidget *button, PRBool *value);
 static void get_editable (GtkWidget *editable, PRUnichar **text);
 
-/**
- * class CPromptService: an GNOME implementation of prompt dialogs for
- * Mozilla
- */
-class CPromptService: public nsIPromptService
-{
-	public:
-		CPromptService();
-  		virtual ~CPromptService();
-
-		NS_DECL_ISUPPORTS
-		NS_DECL_NSIPROMPTSERVICE
-	private:
-		nsresult AddButton (GtkWidget *dialog, 
-		   	   	    char type, 
-		   	   	    const PRUnichar *title,
-		   	   	    int id);
-};
-
 NS_IMPL_ISUPPORTS1 (CPromptService, nsIPromptService)
 
 /**
@@ -220,15 +198,15 @@ NS_IMETHODIMP CPromptService::ConfirmCheck (nsIDOMWindow *parent,
 	return NS_OK;
 }
 
-NS_IMETHODIMP CPromptService::AddButton (GtkWidget *dialog, 
-		   			 char type, 
-		   			 const PRUnichar *title,
-		   			 int id)
+NS_METHOD CPromptService::AddButton(GtkWidget *dialog, 
+		   		    char type, 
+		   		    const PRUnichar *title,
+		   		    int id)
 {
 	const char *btitle;
-	const nsACString &utf8string = NS_ConvertUCS2toUTF8 (title);
+	const nsACString &utf8string = NS_ConvertUCS2toUTF8(title);
 
-	g_print ("%d", type);
+	//g_print("%d", type);
 	
 	switch (type)
 	{
@@ -261,10 +239,10 @@ NS_IMETHODIMP CPromptService::AddButton (GtkWidget *dialog,
 			return NS_ERROR_FAILURE;
 	}
 
-	gtk_dialog_add_button (GTK_DIALOG(dialog),
-			       btitle ? btitle : 
-			       PromiseFlatCString(utf8string).get(),
-			       id);
+	gtk_dialog_add_button(GTK_DIALOG(dialog),
+			      btitle ? btitle : 
+			      PromiseFlatCString(utf8string).get(),
+			      id);
 
 	return NS_OK;
 }
@@ -357,41 +335,49 @@ NS_IMETHODIMP CPromptService::Prompt (nsIDOMWindow *parent,
 	gint ret;
 
 	/* build and show the dialog */
-	gxml = gul_glade_widget_new ("prompts.glade", "prompt_dialog", 
-				 &dialog, NULL);
-	entry = glade_xml_get_widget (gxml, "entry");
-	label = glade_xml_get_widget (gxml, "label");
-	check_button = glade_xml_get_widget (gxml, "check_button");
-	g_object_unref (G_OBJECT (gxml));
+	gxml = gul_glade_widget_new("prompts.glade", "prompt_dialog", 
+				    &dialog, NULL);
+	entry = glade_xml_get_widget(gxml, "entry");
+	label = glade_xml_get_widget(gxml, "label");
+	check_button = glade_xml_get_widget(gxml, "check_button");
+	g_object_unref(G_OBJECT(gxml));
 
 	/* parent the dialog */
-	gparent = MozillaFindGtkParent (parent);
-	gtk_window_set_transient_for (GTK_WINDOW (dialog),
-                                      GTK_WINDOW (gparent));
+	gparent = MozillaFindGtkParent(parent);
+	gtk_window_set_transient_for(GTK_WINDOW(dialog),
+                                     GTK_WINDOW(gparent));
 	
 	/* set dynamic attributes */
-	set_title (dialog, dialogTitle);
-	set_label_text (label, text);
-	set_check_button_text (check_button, checkMsg);
-	set_editable (entry, value);
-	set_check_button (check_button, checkValue);
-	set_check_button_size_to_label (check_button, label);
+	set_title(dialog, dialogTitle);
+	set_label_text(label, text);
+	set_editable(entry, value);
+	if (checkMsg)
+	{
+		set_check_button_text(check_button, checkMsg);
+		set_check_button(check_button, checkValue);
+		set_check_button_size_to_label(check_button, label);
+	}
+	else
+		gtk_widget_hide(check_button);
 	
 	/* make a suitable sound */
 	/* NB: should be question, but this is missing in many
 	 * of the current gnome sound packages that I've tried... */
-	gnome_triggers_vdo ("", "generic", NULL);
+	gnome_triggers_vdo("", "generic", NULL);
 
 	/* run dialog and capture return values */
-	ret = gtk_dialog_run (GTK_DIALOG (dialog));
+	ret = gtk_dialog_run(GTK_DIALOG (dialog));
 
 	/* handle return values */
-	get_check_button (check_button, checkValue);
-	get_editable (entry, value);
+	get_editable(entry, value);
+
+	if (checkMsg)
+		get_check_button(check_button, checkValue);
+
 	*_retval = (ret == GTK_RESPONSE_OK);
 
 	/* done */
-	gtk_widget_destroy (dialog);
+	gtk_widget_destroy(dialog);
 	return NS_OK;
 }
 
@@ -623,27 +609,6 @@ NS_IMETHODIMP CPromptService::Select (nsIDOMWindow *parent,
 	return NS_OK;
 }
 
-NS_DEF_FACTORY (CPromptService, CPromptService);
-
-/**
- * NS_NewPromptServiceFactory:
- */ 
-nsresult NS_NewPromptServiceFactory(nsIFactory** aFactory)
-{
-	NS_ENSURE_ARG_POINTER(aFactory);
-	*aFactory = nsnull;
-
-	nsCPromptServiceFactory *result = new nsCPromptServiceFactory;
-	if (result == NULL)
-	{
-		return NS_ERROR_OUT_OF_MEMORY;
-	}
-    
-	NS_ADDREF(result);
-	*aFactory = result;
-  
-	return NS_OK;
-}
 
 /**
  * set_title: set a dialog title to a unicode string
@@ -716,7 +681,7 @@ set_check_button_size_to_label (GtkWidget *check_button,
 
 	if (r.width <= label_r.width) return;
 	
-	gtk_widget_set_size_request (label, r.width, 0);
+	gtk_widget_set_size_request (label, r.width, -1);
 }
 
 /**
