@@ -19,7 +19,10 @@
 #ifndef __PromptService_h
 #define __PromptService_h
 
-#include "nsError.h"
+#include <gtk/gtkwidget.h>
+
+#include <nsError.h>
+#include <nsIPromptService.h>
 
 #define G_PROMPTSERVICE_CID			     \
 { /* c5a77759-a07a-4025-8f74-ae89153ee6c2 */         \
@@ -32,8 +35,23 @@
 #define G_PROMPTSERVICE_CLASSNAME "Galeon's Prompt Service"
 #define G_PROMPTSERVICE_CONTRACTID "@mozilla.org/embedcomp/prompt-service;1"
 
-class nsIFactory;
+/**
+ * class CPromptService: an GNOME implementation of prompt dialogs for
+ * Mozilla
+ */
+class CPromptService: public nsIPromptService
+{
+	public:
+		CPromptService();
+  		virtual ~CPromptService();
 
-extern nsresult NS_NewPromptServiceFactory(nsIFactory** aFactory);
+		NS_DECL_ISUPPORTS
+		NS_DECL_NSIPROMPTSERVICE
+	private:
+		NS_METHOD AddButton(GtkWidget *dialog, 
+		   	   	    char type, 
+		   	   	    const PRUnichar *title,
+		   	   	    int id);
+};
 
 #endif
