@@ -16,6 +16,10 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include "glib.h"
 #include "js-console.h"
 #include "gul-string.h"
@@ -34,14 +38,16 @@
 #include "nsIConsoleService.h"
 #include "JSConsoleListener.h"
 #include "nsIJSConsoleService.h"
+#include "CookieObserver.h"
 #include "mozilla-cookie.h"
 
 #include <time.h>
-#include <libgnome/gnome-i18n.h>
+#include <bonobo/bonobo-i18n.h>
 #include <string.h>
 #include <nsICacheService.h>
 #include <nsCOMPtr.h>
 #include <nsNetCID.h>
+#include <nsNetUtil.h>
 #include <nsIServiceManager.h>
 #include <nsIIOService.h>
 #include <nsIProtocolProxyService.h>
@@ -60,6 +66,7 @@
 #include <nsICookie.h>
 #include <nsCCookieManager.h>
 #include <nsCPasswordManager.h>
+#include <nsIObserverService.h>
 
 #define MOZILLA_PROFILE_DIR  GALEON_DIR"/mozilla"
 #define MOZILLA_PROFILE_NAME "galeon"
@@ -115,6 +122,11 @@ impl_remove_permissions (GaleonEmbedShell *shell,
 		         PermissionType type,
 		         GList *permissions);
 static gresult           
+impl_test_permission (GaleonEmbedShell *shell,
+		      const char *url, 
+		      PermissionType type,
+		      PermissionActionType *ret_action);
+static gresult
 impl_list_cookies (GaleonEmbedShell *shell,
 		   GList **cookies);
 static gresult           
@@ -209,6 +221,7 @@ mozilla_embed_shell_class_init (MozillaEmbedShellClass *klass)
 	shell_class->set_permission = impl_set_permission;
 	shell_class->list_permissions = impl_list_permissions;
 	shell_class->remove_permissions = impl_remove_permissions;
+	shell_class->test_permission = impl_test_permission;
 	shell_class->list_cookies = impl_list_cookies;
 	shell_class->remove_cookies = impl_remove_cookies;
 	shell_class->list_passwords = impl_list_passwords;
@@ -305,7 +318,7 @@ mozilla_set_default_prefs (void)
 
         /* set default search engine */
         mozilla_prefs_set_string ("keyword.URL",_("http://www.google.com/search?q="));
-        mozilla_prefs_set_boolean ("keyword.enabled", TRUE);
+        mozilla_prefs_set_boolean ("keyword.enabled", FALSE);
         mozilla_prefs_set_boolean ("security.checkloaduri", FALSE); 
 
         /* while we have no UI */
@@ -439,6 +452,7 @@ static void
 mozilla_register_external_protocols (void)
 {
 	/* FIXME register only when needed */
+#if 0
 	if (have_gnome_url_handler ("ftp"))
 	{
 		mozilla_register_FtpProtocolHandler ();
@@ -447,11 +461,35 @@ mozilla_register_external_protocols (void)
 	{
 		mozilla_unregister_FtpProtocolHandler ();
 	}
+#endif
 
 	mozilla_register_MailtoProtocolHandler ();
 	mozilla_register_gHelpProtocolHandlers ();
 }
 
+#if MOZILLA_SNAPSHOT > 6
+static void
+mozilla_init_cookie_observer(void)
+{
+	//We new this observer but never delete it
+	//This is ok as mozilla_embed_shell's lifetime is
+	//the whole time galeon is running. Galeon exits 
+	//when it is destroyed, so there's no real need to
+	//clean up. 
+
+	nsresult rv;
+
+	nsCOMPtr<nsIObserverService> os(do_GetService("@mozilla.org/observer-service;1", &rv));
+	if(NS_FAILED(rv) || ! os) return;
+
+	GCookieObserver* cookieObserver = new GCookieObserver();
+
+	rv = os->AddObserver(cookieObserver,
+			     PERM_CHANGE_NOTIFICATION,
+			     PR_FALSE);
+}
+#endif
+
 static void
 mozilla_embed_shell_init (MozillaEmbedShell *mes)
 {
@@ -490,6 +528,10 @@ mozilla_embed_shell_init (MozillaEmbedShell *mes)
 	mozilla_register_external_protocols ();
 
 	mozilla_init_sidebar ();
+
+#if MOZILLA_SNAPSHOT > 6
+	mozilla_init_cookie_observer();
+#endif
 		
 	/* FIXME alert if fails */
 }
@@ -762,12 +804,12 @@ impl_get_charset_titles (GaleonEmbedShell *shell,
 	for (j = 0; j < count; j++)
 	{	
 		if (group == NULL ||
-		    strcmp (group, lgroups[charset_trans_array[j].lgroup]) == 0) 
+		    strcmp (group, _(lgroups[charset_trans_array[j].lgroup])) == 0) 
 		{
 			CharsetInfo *info;
 			info = g_new0 (CharsetInfo, 1);
 			info->name = charset_trans_array[j].charset_name;
-			info->title = charset_trans_array[j].charset_title;
+			info->title = _(charset_trans_array[j].charset_title);
 			l = g_list_append (l, info);
 
 			/* FIXME check that the encoding exists in mozilla before
@@ -789,7 +831,7 @@ impl_get_charset_groups (GaleonEmbedShell *shell,
 	
 	for (i = 0; lgroups[i] != NULL; i++)
 	{
-		l = g_list_append (l, (gpointer)lgroups[i]);
+		l = g_list_append (l, _(lgroups[i]));
 	}
 	
 	*groups = l;
@@ -848,17 +890,26 @@ impl_get_font_list (GaleonEmbedShell *shell,
 }
 
 static gresult           
-impl_set_permission (GaleonEmbedShell *shell,
-		     const char *url, 
-		     PermissionType type,
-		     gboolean allow)
+impl_set_permission(GaleonEmbedShell *shell,
+		    const char *url, 
+		    PermissionType type,
+		    gboolean allow)
 {
 	nsresult rv;
 	nsCOMPtr<nsIPermissionManager> permissionManager =
-                        do_CreateInstance (NS_PERMISSIONMANAGER_CONTRACTID);
+                        do_GetService(NS_PERMISSIONMANAGER_CONTRACTID);
 
-        rv = permissionManager->Add (nsDependentCString(url),
-        			     allow ? PR_TRUE : PR_FALSE, type);
+#if MOZILLA_SNAPSHOT > 6
+	nsCOMPtr<nsIURI> uri;
+	rv = NS_NewURI(getter_AddRefs(uri), url);
+	if (NS_FAILED(rv) || !uri) return G_FAILED;
+	rv = permissionManager->Add(uri, type,
+				    allow ? (PRUint32)nsIPermissionManager::ALLOW_ACTION
+				    	  : (PRUint32)nsIPermissionManager::DENY_ACTION);
+#else
+        rv = permissionManager->Add(nsDependentCString(url),
+        			    allow ? PR_TRUE : PR_FALSE, type);
+#endif
 	if (NS_FAILED(rv)) return G_FAILED;
 
 	return G_OK;
@@ -874,9 +925,9 @@ impl_list_permissions (GaleonEmbedShell *shell,
 	*permissions = NULL;
 	
 	nsCOMPtr<nsIPermissionManager> permissionManager = 
-                        do_CreateInstance (NS_PERMISSIONMANAGER_CONTRACTID);
+                        do_GetService(NS_PERMISSIONMANAGER_CONTRACTID);
         nsCOMPtr<nsISimpleEnumerator> permissionEnumerator;
-        result = permissionManager->GetEnumerator (getter_AddRefs(permissionEnumerator));
+        result = permissionManager->GetEnumerator(getter_AddRefs(permissionEnumerator));
         if (NS_FAILED(result)) return G_FAILED;
 	
         PRBool enumResult;
@@ -885,80 +936,142 @@ impl_list_permissions (GaleonEmbedShell *shell,
              permissionEnumerator->HasMoreElements(&enumResult))
         {
                 nsCOMPtr<nsIPermission> nsPermission;
-                result = permissionEnumerator->GetNext (getter_AddRefs(nsPermission));
+                result = permissionEnumerator->GetNext(getter_AddRefs(nsPermission));
                 if (NS_FAILED(result)) return G_FAILED;
 
+#if MOZILLA_SNAPSHOT > 6
+		PRUint32 cType;
+#else
                 PRInt32 cType;
-                nsPermission->GetType (&cType);
-                if (cType == type)
+#endif
+                nsPermission->GetType(&cType);
+                if ((PRUint32)cType == (PRUint32)type)
                 {
-                        PermissionInfo *b = g_new0 (PermissionInfo, 1);
-                        gchar *tmp = NULL;
+                        PermissionInfo *b = g_new0(PermissionInfo, 1);
+#if MOZILLA_SNAPSHOT > 6
+			nsCString host;
+			nsPermission->GetHost(host);
+			b->domain = g_strdup(host.get());
 
-                        nsPermission->GetHost (&tmp);
-                        b->domain = g_strdup (tmp);
-                        nsMemory::Free (tmp);
+			PRUint32 cap;
+			nsPermission->GetCapability(&cap);
+			switch (cap)
+			{
+			  case nsIPermissionManager::ALLOW_ACTION :
+			  	b->status = TRUE;
+			  	break;
+			  case nsIPermissionManager::DENY_ACTION :
+			 	b->status = FALSE;
+			 	break;
+			  default :
+			  	b->status = FALSE;
+			}
+#else
+			nsXPIDLCString host;
+                        nsPermission->GetHost(getter_Copies(host));
+                        b->domain = g_strdup(host.get());
 
                         PRBool cap;
-                        nsPermission->GetCapability (&cap);
+                        nsPermission->GetCapability(&cap);
 			b->status = cap;
-
-                        *permissions = g_list_prepend (*permissions, b);
+#endif
+                        *permissions = g_list_prepend(*permissions, b);
                 }
         }
 
-        *permissions = g_list_reverse (*permissions);
+        *permissions = g_list_reverse(*permissions);
 	
 	return G_OK;
 }
 
 static gresult           
-impl_remove_permissions (GaleonEmbedShell *shell,
-		         PermissionType type,
-		         GList *permissions)
+impl_remove_permissions(GaleonEmbedShell *shell,
+		        PermissionType type,
+		        GList *permissions)
 {
 	nsresult result;
         nsCOMPtr<nsIPermissionManager> permissionManager =
-                        do_CreateInstance (NS_PERMISSIONMANAGER_CONTRACTID);
+                        do_GetService(NS_PERMISSIONMANAGER_CONTRACTID);
 
         for (GList *l = permissions; l != NULL; l = l->next)
         {
                 PermissionInfo *b = (PermissionInfo *)l->data;
-                result = permissionManager->Remove (nsDependentCString(b->domain),
-                				    type);
+                result = permissionManager->Remove(nsDependentCString(b->domain),
+                				   type);
                 if (NS_FAILED(result)) return G_FAILED;
         };
 
 	return G_OK;
 }
 
+static gresult           
+impl_test_permission (GaleonEmbedShell *shell,
+		      const char *url, 
+		      PermissionType type,
+		      PermissionActionType *ret_action)
+{
+	nsresult rv;
+	nsCOMPtr<nsIPermissionManager> permissionManager =
+                        do_GetService(NS_PERMISSIONMANAGER_CONTRACTID);
+
+#if MOZILLA_SNAPSHOT > 6
+	nsCOMPtr<nsIURI> uri;
+	rv = NS_NewURI(getter_AddRefs(uri), url);
+	if (NS_FAILED(rv) || !uri) return G_FAILED;
+
+	PRUint32 action;
+	rv = permissionManager->TestPermission(uri, type, &action);
+
+	switch (action)
+	{
+	  case nsIPermissionManager::UNKNOWN_ACTION :
+	  	*ret_action = UNKNOWN_ACTION;
+	  	break;
+	  case nsIPermissionManager::ALLOW_ACTION :
+	  	*ret_action = ALLOW_ACTION;
+	  	break;
+	  case nsIPermissionManager::DENY_ACTION :
+	  	*ret_action = DENY_ACTION;
+	  	break;
+	}
+#else
+	PRBool allow;
+        rv = permissionManager->TestForBlocking(nsDependentCString(url),
+					        type, &allow);
+
+	*ret_action = allow ? ALLOW_ACTION : DENY_ACTION;
+#endif
+	if (NS_FAILED(rv)) return G_FAILED;
+
+	return G_OK;
+}
+
 static gresult           
 impl_list_cookies (GaleonEmbedShell *shell,
 		   GList **cookies)
 {
         nsresult result;
-
         nsCOMPtr<nsICookieManager> cookieManager = 
-                        do_CreateInstance (NS_COOKIEMANAGER_CONTRACTID);
+                        do_GetService(NS_COOKIEMANAGER_CONTRACTID);
         nsCOMPtr<nsISimpleEnumerator> cookieEnumerator;
         result = 
-            cookieManager->GetEnumerator (getter_AddRefs(cookieEnumerator));
+            cookieManager->GetEnumerator(getter_AddRefs(cookieEnumerator));
         if (NS_FAILED(result)) return G_FAILED;
-	
+
         PRBool enumResult;
         for (cookieEnumerator->HasMoreElements(&enumResult) ;
              enumResult == PR_TRUE ;
              cookieEnumerator->HasMoreElements(&enumResult))
         {
                 nsCOMPtr<nsICookie> nsCookie;
-                result = cookieEnumerator->GetNext (getter_AddRefs(nsCookie));
+                result = cookieEnumerator->GetNext(getter_AddRefs(nsCookie));
                 if (NS_FAILED(result)) return G_FAILED;
 
-                CookieInfo *c = mozilla_cookie_to_info (nsCookie);
-                *cookies = g_list_prepend (*cookies, c);
+                CookieInfo *c = mozilla_cookie_to_info(nsCookie);
+                *cookies = g_list_prepend(*cookies, c);
         }       
 
-	*cookies = g_list_reverse (*cookies);
+	*cookies = g_list_reverse(*cookies);
 		
 	return G_OK;
 }
@@ -969,15 +1082,15 @@ impl_remove_cookies (GaleonEmbedShell *shell,
 {
 	nsresult result;
         nsCOMPtr<nsICookieManager> cookieManager =
-                        do_CreateInstance (NS_COOKIEMANAGER_CONTRACTID);
+                        do_GetService(NS_COOKIEMANAGER_CONTRACTID);
 	
         for (GList *l = cookies; l != NULL; l = l->next)
         {
                 CookieInfo *c = (CookieInfo *)l->data;
 
-                result = cookieManager->Remove (NS_LITERAL_CSTRING(c->domain),
-                                                NS_LITERAL_CSTRING(c->name),
-                                                NS_LITERAL_CSTRING(c->path),
+                result = cookieManager->Remove (nsDependentCString(c->domain),
+                                                nsDependentCString(c->name),
+                                                nsDependentCString(c->path),
                                                 PR_FALSE);
                 if (NS_FAILED(result)) return G_FAILED;
         };
@@ -993,7 +1106,7 @@ impl_list_passwords (GaleonEmbedShell *shell,
         nsresult result = NS_ERROR_FAILURE;
 
         nsCOMPtr<nsIPasswordManager> passwordManager =
-                        do_CreateInstance (NS_PASSWORDMANAGER_CONTRACTID);
+                        do_GetService(NS_PASSWORDMANAGER_CONTRACTID);
         nsCOMPtr<nsISimpleEnumerator> passwordEnumerator;
         if (type == PASSWORD_PASSWORD)
                 result = passwordManager->GetEnumerator 
@@ -1041,14 +1154,14 @@ impl_remove_passwords (GaleonEmbedShell *shell,
 {
 	nsresult result = NS_ERROR_FAILURE;
         nsCOMPtr<nsIPasswordManager> passwordManager =
-                        do_CreateInstance (NS_PASSWORDMANAGER_CONTRACTID);
+                        do_GetService(NS_PASSWORDMANAGER_CONTRACTID);
 
         for (GList *l = passwords; l != NULL; l = l->next)
         {
                 PasswordInfo *p = (PasswordInfo *)l->data;
                 if (type == PASSWORD_PASSWORD)
                 {
-                        result = passwordManager->RemoveUser (NS_LITERAL_CSTRING(p->host),
+                        result = passwordManager->RemoveUser (nsDependentCString(p->host),
                                                               NS_ConvertUTF8toUCS2(nsDependentCString(p->username)));
                 }
                 else if (type == PASSWORD_REJECT)
@@ -1129,8 +1242,6 @@ impl_show_file_picker (GaleonEmbedShell *shell,
         }
         else
         {
-                if (*ret_fullpath)
-                        g_free (*ret_fullpath);
                 nsCOMPtr<nsILocalFile> file;
                 filePicker->GetFile (getter_AddRefs(file));
 		nsCAutoString tempFullPathStr;
