@@ -20,6 +20,7 @@
 #include "gtkmozembed.h"
 #include "gtkmozembed_internal.h"
 #include "gul-string.h"
+#include "gul-gobject-misc.h"
 #include "galeon-embed.h"
 #include "mozilla-embed.h"
 #include "MozillaPrivate.h"
@@ -32,12 +33,18 @@
 #include <nsString.h>
 #include <nsIRequest.h>
 #include <nsIWebProgressListener.h>
+#include <nsIWindowWatcher.h>
 #include <nsITransportSecurityInfo.h>
 #include <nsIPrintOptions.h>
 #include <nsGfxCIID.h>
 
 #include <math.h>
 
+#define WINDOWWATCHER_CONTRACTID "@mozilla.org/embedcomp/window-watcher;1"
+
+//#define DEBUG_MSG(x)
+#define DEBUG_MSG(x) g_print x
+
 static void
 mozilla_embed_class_init (MozillaEmbedClass *klass);
 static void
@@ -105,9 +112,6 @@ impl_copy_page (GaleonEmbed *dest,
 static gresult 
 impl_grab_focus (GaleonEmbed *embed);
 static gresult
-impl_get_favicon_location (GaleonEmbed *embed,
-			   char **url);
-static gresult
 impl_get_link_tags (GaleonEmbed *embed,
                     const char *link_type,
                     GList **tags);
@@ -149,7 +153,10 @@ impl_shistory_go_nth (GaleonEmbed *embed,
                       int nth);
 static gboolean
 impl_shistory_copy (GaleonEmbed *source,
-                    GaleonEmbed *dest);
+		    GaleonEmbed *dest,
+		    gboolean back_history,
+		    gboolean forward_history,
+		    gboolean set_current);
 static gresult
 impl_scroll (GaleonEmbed *embed, 
              EmbedScrollDirection direction);
@@ -216,10 +223,12 @@ static void
 mozilla_embed_new_window_cb (GtkMozEmbed *embed, 
 			     GtkMozEmbed **newEmbed,
                              guint chromemask, MozillaEmbed *membed);
+
 static void
 mozilla_embed_security_change_cb (GtkMozEmbed *embed, 
 				  gpointer request,
                                   guint state, MozillaEmbed *membed);
+
 static EmbedSecurityLevel
 mozilla_embed_security_level (MozillaEmbed *membed);
 
@@ -251,56 +260,24 @@ signal_connections[] =
 
 struct MozillaEmbedPrivate
 {
+	MozillaEmbedPrivate() : wrapper(NULL), security_state(-1), no_page(1)
+	{ /* nothing */ }
+
 	GaleonWrapper *wrapper;
-	nsIRequest *request;
+	nsCOMPtr<nsIRequest> request;
 	gint security_state;
 
 	/* HACK 1: No page loaded, 0: Loading an empty page, -1: Page loaded */ 
-	gboolean no_page;
+	gint no_page;
 };
 
 static NS_DEFINE_CID(kPrintOptionsCID, NS_PRINTOPTIONS_CID);
 
 static GObjectClass *parent_class = NULL;
 
-GType 
-mozilla_embed_get_type (void)
-{
-        static GType mozilla_embed_type = 0;
-
-        if (mozilla_embed_type == 0)
-        {
-                static const GTypeInfo our_info =
-                {
-                        sizeof (MozillaEmbedClass),
-                        NULL, /* base_init */
-                        NULL, /* base_finalize */
-                        (GClassInitFunc) mozilla_embed_class_init,
-                        NULL,
-                        NULL, /* class_data */
-                        sizeof (MozillaEmbed),
-                        0, /* n_preallocs */
-                        (GInstanceInitFunc) mozilla_embed_init
-                };
-
-		static const GInterfaceInfo embed_info =
-		{
-			(GInterfaceInitFunc) galeon_embed_init,    /* interface_init */
-        		NULL,                                      /* interface_finalize */
-        		NULL                                       /* interface_data */
-     		 };
-		
-                mozilla_embed_type = g_type_register_static (GTK_TYPE_MOZ_EMBED,
-							     "MozillaEmbed",
-							     &our_info, 
-							     (GTypeFlags)0);
-		g_type_add_interface_static (mozilla_embed_type,
-                                   	     GALEON_EMBED_TYPE,
-                                   	     &embed_info);
-        }
-
-        return mozilla_embed_type;
-}
+MAKE_GET_TYPE_IFACE (mozilla_embed, "MozillaEmbed", MozillaEmbed,
+		     mozilla_embed_class_init, mozilla_embed_init, GTK_TYPE_MOZ_EMBED,
+		     galeon_embed_init, GALEON_EMBED_TYPE);
 
 static void
 galeon_embed_init (GaleonEmbedClass *embed_class)
@@ -324,7 +301,6 @@ galeon_embed_init (GaleonEmbedClass *embed_class)
 	embed_class->reload = impl_reload;
 	embed_class->copy_page = impl_copy_page;
 	embed_class->grab_focus = impl_grab_focus;
-	embed_class->get_favicon_location = impl_get_favicon_location;
 	embed_class->get_link_tags = impl_get_link_tags;
 	embed_class->zoom_set = impl_zoom_set;
 	embed_class->zoom_get = impl_zoom_get;
@@ -365,12 +341,8 @@ mozilla_embed_class_init (MozillaEmbedClass *klass)
 static void
 mozilla_embed_init (MozillaEmbed *embed)
 {
-        embed->priv = g_new0 (MozillaEmbedPrivate, 1);
-	embed->priv->wrapper = NULL;
-	embed->priv->request = NULL;
-	embed->priv->security_state = -1;
-	embed->priv->no_page = 1;
-	
+        embed->priv = new MozillaEmbedPrivate;
+
 	mozilla_embed_connect_signals (embed);
 }
 
@@ -423,11 +395,11 @@ mozilla_embed_finalize (GObject *object)
 	if (embed->priv->wrapper)
 	{
 		embed->priv->wrapper->Destroy();
-        	embed->priv->wrapper = NULL;
         	delete embed->priv->wrapper;
+        	embed->priv->wrapper = NULL;
 	}
 
-        g_free (embed->priv);
+	delete embed->priv;
 	
         G_OBJECT_CLASS (parent_class)->finalize (object);
 
@@ -439,8 +411,6 @@ mozilla_embed_finalize (GObject *object)
 gpointer
 mozilla_embed_get_galeon_wrapper (MozillaEmbed *embed)
 {
-	g_return_val_if_fail (embed->priv->wrapper != NULL, NULL);
-	
 	return embed->priv->wrapper;
 }
 
@@ -525,7 +495,7 @@ impl_can_go_up (GaleonEmbed *embed)
 	gresult result;
 
 	if (galeon_embed_get_location (embed, TRUE, FALSE, &location) != G_OK)
-	  return G_FAILED;
+		return G_FAILED;
 	g_return_val_if_fail (location != NULL, G_FAILED);
 	if ((s = mozilla_embed_get_uri_parent (location)) != NULL)
 	{
@@ -587,7 +557,8 @@ impl_go_up (GaleonEmbed *embed)
 	char *uri;
 	char *parent_uri;
 	
-	galeon_embed_get_location (embed, TRUE, FALSE, &uri);
+	if (galeon_embed_get_location (embed, TRUE, FALSE, &uri) != G_OK)
+		return G_FAILED;
 	g_return_val_if_fail (uri != NULL, G_FAILED);
 	
 	parent_uri = mozilla_embed_get_uri_parent (uri);
@@ -601,23 +572,23 @@ impl_go_up (GaleonEmbed *embed)
 }
 
 static char *
-mozilla_embed_get_uri_parent (const char *uri)
+mozilla_embed_get_uri_parent (const char *aUri)
 {
         nsresult rv;
 
-        nsCOMPtr<nsIURI> aURI;
-        rv = NS_NewURI (getter_AddRefs(aURI), uri);
-        if (NS_FAILED(rv) || !aURI) return NULL;
+        nsCOMPtr<nsIURI> uri;
+        rv = NS_NewURI (getter_AddRefs(uri), aUri);
+        if (NS_FAILED(rv) || !uri) return NULL;
 
-        nsCOMPtr<nsIURL> aURL = do_QueryInterface(aURI, &rv);
-        if (NS_FAILED(rv) || !aURL) return NULL;
+        nsCOMPtr<nsIURL> url = do_QueryInterface(uri, &rv);
+        if (NS_FAILED(rv) || !url) return NULL;
 
         nsCAutoString dirPath;
-        rv = aURL->GetDirectory (dirPath);
+        rv = url->GetDirectory (dirPath);
         if (NS_FAILED(rv) || !dirPath.Length()) return NULL;
 
         nsCAutoString filePath;
-        rv = aURL->GetFilePath (filePath);
+        rv = url->GetFilePath (filePath);
         if (NS_FAILED(rv) || !filePath.Length()) return NULL;
 
         PRInt32 pathLength = filePath.Length();
@@ -630,18 +601,18 @@ mozilla_embed_get_uri_parent (const char *uri)
 
         if(trailingSlash != (pathLength-1))
         {
-                aURI->SetPath(dirPath);
+                uri->SetPath(dirPath);
         }
         else
         {
                 PRInt32 nextSlash = filePath.RFind("/",PR_FALSE,trailingSlash-1);
                 nsCAutoString parentPath;
                 filePath.Left(parentPath, nextSlash);
-                aURI->SetPath(parentPath);
+                uri->SetPath(parentPath);
         }
 
         nsCAutoString spec;
-        aURI->GetSpec(spec);
+        uri->GetSpec(spec);
 
         return !spec.IsEmpty() ? g_strdup(spec.get()) : NULL;
 }
@@ -764,14 +735,17 @@ impl_reload (GaleonEmbed *embed,
 	    (flags & EMBED_RELOAD_BYPASSPROXY))
 	{
 		mflags = GTK_MOZ_EMBED_FLAG_RELOADBYPASSPROXYANDCACHE;
+		DEBUG_MSG (("Should bypass cache and proxy\n"));
 	}
 	else if (flags & EMBED_RELOAD_BYPASSCACHE)
 	{
 		mflags = GTK_MOZ_EMBED_FLAG_RELOADBYPASSCACHE;
+		DEBUG_MSG (("Should bypass cache\n"));
 	}
 	else if (flags & EMBED_RELOAD_BYPASSPROXY)
 	{
 		mflags = GTK_MOZ_EMBED_FLAG_RELOADBYPASSPROXY;
+		DEBUG_MSG (("Should bypass proxy\n"));
 	}
 	
 	gtk_moz_embed_reload (GTK_MOZ_EMBED(embed),
@@ -787,6 +761,14 @@ impl_copy_page (GaleonEmbed *dest,
 {
 	GaleonWrapper *dWrapper;
 	dWrapper = MOZILLA_EMBED(dest)->priv->wrapper;
+	if (dWrapper == NULL)
+	{
+		// We need to load some page to get the wrapper initialized.
+		// That's an implementation specific issue the caller doesn't
+		// need to know.
+		gtk_moz_embed_load_url (GTK_MOZ_EMBED(dest), "about:blank");
+		dWrapper = MOZILLA_EMBED(dest)->priv->wrapper;
+	}
 	g_return_val_if_fail (dWrapper != NULL, G_FAILED);
 
 	GaleonWrapper *sWrapper;
@@ -795,11 +777,11 @@ impl_copy_page (GaleonEmbed *dest,
 
         nsresult rv;
 
-        nsCOMPtr<nsISupports> aPageDescriptor;
-        rv = sWrapper->GetPageDescriptor(getter_AddRefs(aPageDescriptor));
-        if (!aPageDescriptor || NS_FAILED(rv)) return G_FAILED;
+        nsCOMPtr<nsISupports> pageDescriptor;
+        rv = sWrapper->GetPageDescriptor(getter_AddRefs(pageDescriptor));
+        if (!pageDescriptor || NS_FAILED(rv)) return G_FAILED;
 
-        rv = dWrapper->LoadDocument(aPageDescriptor, static_cast<PRUint32>(display_type));
+        rv = dWrapper->LoadDocument(pageDescriptor, static_cast<PRUint32>(display_type));
         if (NS_FAILED(rv)) return G_FAILED;
 
         return G_OK;
@@ -821,25 +803,6 @@ impl_get_link_tags (GaleonEmbed *embed,
 	return G_NOT_IMPLEMENTED;
 }
 
-static gresult
-impl_get_favicon_location (GaleonEmbed *embed,
-			   char **url)
-{
-	GaleonWrapper *wrapper;
-	
-	wrapper = MOZILLA_EMBED(embed)->priv->wrapper;
-	g_return_val_if_fail (wrapper != NULL, G_FAILED);
-
-	nsCAutoString transfer;
-	nsresult rv = wrapper->GetFaviconURL (transfer);
-	if (transfer.IsEmpty())
-		*url = NULL;
-	else
-		*url = g_strdup (transfer.get());
-	
-	return NS_SUCCEEDED (rv) ? G_OK : G_FAILED;
-}
-
 static gresult
 impl_zoom_set (GaleonEmbed *embed, 
                int zoom, 
@@ -1009,12 +972,10 @@ static gresult
 impl_shistory_get_nth (GaleonEmbed *embed, 
                        int nth,
                        gboolean is_relative,
-                       char **url,
-                       char **title)
+                       char **aUrl,
+                       char **aTitle)
 {
 	nsresult rv;
-        nsCAutoString aurl;
-	PRUnichar *atitle;
 	GaleonWrapper *wrapper;
 	
 	wrapper = MOZILLA_EMBED(embed)->priv->wrapper;
@@ -1035,13 +996,14 @@ impl_shistory_get_nth (GaleonEmbed *embed,
 		}
 	}
 	
-        rv = wrapper->GetSHUrlAtIndex(nth, aurl);
+        nsCAutoString url;
+        rv = wrapper->GetSHUrlAtIndex(nth, url);
+        *aUrl = (NS_SUCCEEDED (rv) && !url.IsEmpty()) ? g_strdup(url.get()) : NULL;
 
-        *url = (NS_SUCCEEDED (rv) && !aurl.IsEmpty()) ? g_strdup(aurl.get()) : NULL;
-
-	rv = wrapper->GetSHTitleAtIndex(nth, &atitle);
-
-	*title = g_strdup (NS_ConvertUCS2toUTF8(atitle).get());
+	PRUnichar *title;
+	rv = wrapper->GetSHTitleAtIndex(nth, &title);
+	*aTitle = g_strdup (NS_ConvertUCS2toUTF8(title).get());
+	nsMemory::Free(title);
 
 	return G_OK;
 }
@@ -1081,7 +1043,10 @@ impl_shistory_go_nth (GaleonEmbed *embed,
 
 static gboolean
 impl_shistory_copy (GaleonEmbed *source,
-                    GaleonEmbed *dest)
+		    GaleonEmbed *dest,
+		    gboolean back_history,
+		    gboolean forward_history,
+		    gboolean set_current)
 {
 	nsresult rv;
 	GaleonWrapper *s_wrapper;
@@ -1093,7 +1058,7 @@ impl_shistory_copy (GaleonEmbed *source,
 	d_wrapper = MOZILLA_EMBED(dest)->priv->wrapper;
 	g_return_val_if_fail (d_wrapper != NULL, G_FAILED);
 
-	rv = s_wrapper->CopyHistoryTo (d_wrapper);
+	rv = s_wrapper->CopyHistoryTo (d_wrapper, back_history, forward_history, set_current);
 
 	return NS_SUCCEEDED(rv) ? G_OK : G_FAILED;
 }
@@ -1153,22 +1118,23 @@ impl_get_security_level (GaleonEmbed *embed,
 
         nsCOMPtr<nsISupports> info;
         result = channel->GetSecurityInfo(getter_AddRefs(info));
-        if (NS_FAILED (result) || !info) return G_FAILED;
+        if (NS_FAILED (result)) return G_FAILED;
 
-        nsCOMPtr<nsITransportSecurityInfo> secInfo(do_QueryInterface(info));
-        if (!secInfo) return G_FAILED;
+	*description = NULL;
+	if (info)
+	{
+		nsCOMPtr<nsITransportSecurityInfo> secInfo(do_QueryInterface(info));
+		if (!secInfo) return G_FAILED;
 
-        nsXPIDLString tooltip;
-        result = secInfo->GetShortSecurityDescription(getter_Copies(tooltip));
-        if (NS_FAILED (result) || !tooltip) return G_FAILED;
+		nsXPIDLString tooltip;
+		result = secInfo->GetShortSecurityDescription(getter_Copies(tooltip));
+		if (NS_FAILED (result)) return G_FAILED;
 
-        const nsString &string = nsString(tooltip);
-	char *tmp;
-	tmp = ToNewCString (string);
-	
-        *description = g_strdup (tmp);
-
-	nsMemory::Free (tmp);
+		if (tooltip)
+		{
+			*description = g_strdup (NS_ConvertUCS2toUTF8(tooltip).get());
+		}
+	}
 	
 	*level = mozilla_embed_security_level (MOZILLA_EMBED (embed));
 	return G_OK;
@@ -1252,6 +1218,7 @@ impl_find (GaleonEmbed *embed,
         PRBool didFind;
 
         result = wrapper->Find ((NS_ConvertUTF8toUCS2(info->search_string)).get(),
+        			info->interactive,
 				info->match_case, 
                  	        info->backwards, info->wrap,
                        	        info->entire_word, info->search_frames,
@@ -1264,20 +1231,20 @@ static gresult
 impl_set_charset (GaleonEmbed *embed,
                   const char *charset)
 {
-	nsresult result = NS_OK;
+	nsresult rv;
 	GaleonWrapper *wrapper;
-	char *cset;
 
-	cset = g_strdup (charset);
-	
 	wrapper = MOZILLA_EMBED(embed)->priv->wrapper;
 	g_return_val_if_fail (wrapper != NULL, G_FAILED);
 
-	wrapper->ForceCharacterSet (cset);
-
-	g_free (cset);
+	rv = wrapper->ForceCharacterSet (charset);
+	if (NS_SUCCEEDED (rv))
+	{
+		gtk_moz_embed_reload (GTK_MOZ_EMBED (embed),
+				      GTK_MOZ_EMBED_FLAG_RELOADCHARSETCHANGE);
+	}
 	
-	return NS_SUCCEEDED(result) ? G_OK : G_FAILED;
+	return NS_SUCCEEDED(rv) ? G_OK : G_FAILED;
 }
 
 static void
@@ -1380,7 +1347,7 @@ mozilla_embed_link_message_cb (GtkMozEmbed *embed,
 	*getter_Copies(message) = gtk_moz_embed_get_link_message_unichar (embed);
 	
 	g_signal_emit_by_name (membed, "ge_link_message",
-			       g_strdup(NS_ConvertUCS2toUTF8(message).get()));
+			       NS_ConvertUCS2toUTF8(message).get());
 }
 
 static void
@@ -1392,7 +1359,7 @@ mozilla_embed_js_status_cb (GtkMozEmbed *embed,
 	*getter_Copies(status) = gtk_moz_embed_get_js_status_unichar (embed);
 	
 	g_signal_emit_by_name (membed, "ge_js_status",
-			       g_strdup(NS_ConvertUCS2toUTF8(status).get()));
+			       NS_ConvertUCS2toUTF8(status).get());
 }
 
 static void
@@ -1400,6 +1367,20 @@ mozilla_embed_visibility_cb (GtkMozEmbed *embed, gboolean visibility,
 			     MozillaEmbed *membed)
 {
 	g_signal_emit_by_name (membed, "ge_visibility", visibility); 
+
+	GaleonWrapper *wrapper = MOZILLA_EMBED(membed)->priv->wrapper;
+	if (wrapper == NULL) return;
+
+	nsresult rv;
+	nsCOMPtr<nsIWindowWatcher> wwatch
+		(do_GetService(WINDOWWATCHER_CONTRACTID, &rv));
+	if (NS_FAILED(rv) || !wwatch) return;
+
+	nsCOMPtr<nsIDOMWindow> domWindow;
+	rv = wrapper->GetDOMWindow(getter_AddRefs(domWindow));
+	if(NS_FAILED(rv) || !domWindow) return;
+
+	rv = wwatch->SetActiveWindow(domWindow);
 }
 
 static void
@@ -1419,11 +1400,11 @@ mozilla_embed_dom_mouse_click_cb (GtkMozEmbed *embed, gpointer dom_event,
 	GaleonWrapper *wrapper;
 	nsresult result;
 
-	info = galeon_embed_event_new ();
-	
 	wrapper = MOZILLA_EMBED(membed)->priv->wrapper;
 	g_return_val_if_fail (wrapper != NULL, G_FAILED);
 	
+	info = galeon_embed_event_new ();
+	
 	event_context.Init ((nsIDOMEvent*)dom_event, wrapper);
         result = event_context.GetMouseEventInfo (info);
 
@@ -1453,11 +1434,11 @@ mozilla_embed_dom_mouse_down_cb (GtkMozEmbed *embed, gpointer dom_event,
 	GaleonWrapper *wrapper;
 	nsresult result;
 	
-	info = galeon_embed_event_new ();
-	
 	wrapper = MOZILLA_EMBED(membed)->priv->wrapper;
 	g_return_val_if_fail (wrapper != NULL, G_FAILED);
 	
+	info = galeon_embed_event_new ();
+	
 	event_context.Init ((nsIDOMEvent*)dom_event, wrapper);
         result = event_context.GetMouseEventInfo (info);
 	if (NS_SUCCEEDED(result))
