@@ -344,26 +344,18 @@ galeon_init_services (GaleonShell *gs)
 
 }
 
-static char *
-build_homepage_url (GaleonShell *gs,
-		    GaleonEmbed *previous_embed)
+static void
+load_homepage (GaleonShell *gs, 
+	       GaleonEmbed *embed, 
+	       GaleonEmbed *previous_embed)
 {
         const gchar *last_page_url;
         gchar *home_page_url;
         gint page_type;
-        GlobalHistory *gh;
 	char *result = NULL;
 	
         /* find out where we're supposed to start */
-        if (previous_embed == NULL)
-        {
-                page_type = eel_gconf_get_integer 
-                        (CONF_GENERAL_STARTPAGE_TYPE);
-        }
-        else
-        {
-                page_type = eel_gconf_get_integer (CONF_GENERAL_NEWPAGE_TYPE);
-        }
+	page_type = eel_gconf_get_integer (CONF_GENERAL_NEWPAGE_TYPE);
 
         /* return the appropriate page */
         if (page_type == STARTPAGE_HOME)
@@ -374,21 +366,20 @@ build_homepage_url (GaleonShell *gs,
         }
 	else if (page_type == STARTPAGE_LAST)
 	{
+		GlobalHistory *gh;
+
 		if (previous_embed != NULL)
 		{
-			galeon_embed_get_location (previous_embed,
-						   TRUE, TRUE, 
-						   &result);
+			galeon_embed_copy_page (embed, previous_embed, 
+					        DISPLAY_NORMAL);
+			return;
 		}
 
-		if (result == NULL)
-		{
-        		/* get location of last page */
-        		gh = galeon_embed_shell_get_global_history 
-				(gs->priv->embed_shell);
-        		last_page_url = global_history_get_last_page (gh);
-			result = g_strdup (last_page_url);
-		}
+		/* get location of last page */
+		gh = galeon_embed_shell_get_global_history 
+			(gs->priv->embed_shell);
+		last_page_url = global_history_get_last_page (gh);
+		result = g_strdup (last_page_url);
 	}
 
 	if (result == NULL)
@@ -397,7 +388,9 @@ build_homepage_url (GaleonShell *gs,
         	result = g_strdup ("about:blank");
 	}
 
-	return result;
+	galeon_embed_load_url (embed, result);
+
+	g_free (result);
 }
 
 /**
@@ -484,14 +477,7 @@ galeon_shell_new_tab (GaleonShell *shell,
 
 	if (flags & GALEON_NEW_TAB_HOMEPAGE)
 	{
-		char *homepage;
-		
-		homepage = build_homepage_url (shell, previous_embed);
-		g_assert (homepage != NULL);
-
-		galeon_embed_load_url (embed, homepage);
-		
-		g_free (homepage);
+		load_homepage (shell, embed, previous_embed);
 	}
 	else if ((flags & GALEON_NEW_TAB_IS_A_COPY) ||
 		 (flags & GALEON_NEW_TAB_VIEW_SOURCE))
@@ -499,15 +485,19 @@ galeon_shell_new_tab (GaleonShell *shell,
 		EmbedDisplayType display_type =
 			(flags & GALEON_NEW_TAB_VIEW_SOURCE) ?
 			 DISPLAY_AS_SOURCE : DISPLAY_NORMAL;
-		GaleonEmbed *source = galeon_tab_get_embed(previous_tab);
-		galeon_embed_load_url (embed, "about:blank");
-		galeon_embed_copy_page (embed, source, display_type);
+		g_return_val_if_fail (previous_embed != NULL, tab);
+		galeon_embed_copy_page (embed, previous_embed, display_type);
 	}
 	else if (url)
 	{
 		galeon_embed_load_url (embed, url);
 	}
 
+	if (embed && previous_embed)
+	{
+		galeon_embed_shistory_copy (previous_embed, embed, TRUE, FALSE, FALSE);
+	}
+
         return tab;
 }
 
