@@ -47,8 +47,8 @@ void
 history_time_optionmenu_changed_cb (GtkWidget *widget, 
 			            HistoryDialog *dialog);
 void
-history_entry_activate_cb (GtkWidget *widget, 
-			   HistoryDialog *dialog);
+history_entry_changed_cb (GtkWidget *widget, 
+			  HistoryDialog *dialog);
 void
 history_ok_button_clicked_cb (GtkWidget *button,
 			      HistoryDialog *dialog);
@@ -228,7 +228,7 @@ each_url_get_data_binder (GaleonDragEachSelectedItemDataGet iteratee,
 		iteratee (url, -1, -1, -1, -1, data);
 	}
 
-	g_list_free (refs);
+	gul_gui_treeview_free_selection_refs (refs);
 }
 
 static void
@@ -451,15 +451,15 @@ history_dialog_add_url_items (HistoryDialog *dialog,
 static void
 history_dialog_fill_view (HistoryDialog *dialog) 
 {
-	GList *list;
+	GList *list, *l;
 	GtkTreeIter *iter;
 	
 	list = global_history_get_host_list (dialog->priv->gh);
 
-	for (; list != NULL; list = list->next)
+	for (l = list; l != NULL; l = l->next)
 	{
 		GList *urls;
-		HistoryItem *item = (HistoryItem *)(list->data);
+		HistoryItem *item = (HistoryItem *)(l->data);
 
 		urls = global_history_get_urls_list (dialog->priv->gh,
 						     item, 
@@ -766,8 +766,8 @@ history_host_checkbutton_toggled_cb (GtkWidget *widget,
 }
 
 void
-history_entry_activate_cb (GtkWidget *widget, 
-			   HistoryDialog *dialog)
+history_entry_changed_cb (GtkWidget *widget, 
+			  HistoryDialog *dialog)
 {
 	if (dialog->priv->treeview == NULL) return;
 	history_dialog_clear_view (dialog);
