@@ -61,7 +61,6 @@ struct PdmActionInfo
 	GtkWidget *treeview;
 	GList *list;
 	int remove_id;
-	int clear_id;
 	int data_col;
 	PdmDialog *dialog;
 };
@@ -71,18 +70,30 @@ struct PdmDialogPrivate
 	GtkTreeModel *model;
 	PdmActionInfo *cookies;
 	PdmActionInfo *passwords;
+	PdmActionInfo *password_sites;
+	PdmActionInfo *cookie_sites;
+	PdmActionInfo *image_sites;
 };
 
 enum
 {
+	PROP_DIALOG,
+
 	PROP_COOKIES_TREEVIEW,
 	PROP_COOKIES_REMOVE,
-	PROP_COOKIES_CLEAR,
+	PROP_COOKIES_PROPERTIES,
+
 	PROP_PASSWORDS_TREEVIEW,
 	PROP_PASSWORDS_REMOVE,
-	PROP_PASSWORDS_CLEAR,
-	PROP_DIALOG,
-	PROP_COOKIES_PROPERTIES
+
+	PROP_COOKIE_SITES_TREEVIEW,
+	PROP_COOKIE_SITES_REMOVE,
+
+	PROP_IMAGE_SITES_TREEVIEW,
+	PROP_IMAGE_SITES_REMOVE,
+
+	PROP_PASSWORD_SITES_TREEVIEW,
+	PROP_PASSWORD_SITES_REMOVE,
 };
 
 enum
@@ -99,17 +110,47 @@ enum
 	COL_PASSWORDS_DATA
 };
 
+enum
+{
+	COL_COOKIE_SITES_DOMAIN,
+	COL_COOKIE_SITES_STATUS,
+	COL_COOKIE_SITES_DATA
+};
+
+enum
+{
+	COL_IMAGE_SITES_DOMAIN,
+	COL_IMAGE_SITES_STATUS,
+	COL_IMAGE_SITES_DATA
+};
+
+enum
+{
+	COL_PASSWORD_SITES_HOST,
+	COL_PASSWORD_SITES_STATUS,
+	COL_PASSWORD_SITES_DATA
+};
+
 static const
 GaleonDialogProperty properties [] =
 {
+	{ PROP_DIALOG, "pdm_dialog", NULL, PT_NORMAL, NULL },
+
 	{ PROP_COOKIES_TREEVIEW, "cookies_treeview", NULL, PT_NORMAL, NULL },
 	{ PROP_COOKIES_REMOVE, "cookies_remove_button", NULL, PT_NORMAL, NULL },
-	{ PROP_COOKIES_CLEAR, "cookies_clear_button", NULL, PT_NORMAL, NULL },
+	{ PROP_COOKIES_PROPERTIES, "cookies_properties_button", NULL, PT_NORMAL, NULL },
+
 	{ PROP_PASSWORDS_TREEVIEW, "passwords_treeview", NULL, PT_NORMAL, NULL },
 	{ PROP_PASSWORDS_REMOVE, "passwords_remove_button", NULL, PT_NORMAL, NULL },
-	{ PROP_PASSWORDS_CLEAR, "passwords_clear_button", NULL, PT_NORMAL, NULL },
-	{ PROP_DIALOG, "pdm_dialog", NULL, PT_NORMAL, NULL },
-	{ PROP_COOKIES_PROPERTIES, "cookies_properties_button", NULL, PT_NORMAL, NULL },
+
+	{ PROP_COOKIE_SITES_TREEVIEW, "cookie_sites_treeview", NULL, PT_NORMAL, NULL },
+	{ PROP_COOKIE_SITES_REMOVE, "cookie_sites_remove_button", NULL, PT_NORMAL, NULL },
+
+	{ PROP_IMAGE_SITES_TREEVIEW, "image_sites_treeview", NULL, PT_NORMAL, NULL },
+	{ PROP_IMAGE_SITES_REMOVE, "image_sites_remove_button", NULL, PT_NORMAL, NULL },
+
+	{ PROP_PASSWORD_SITES_TREEVIEW, "password_sites_treeview", NULL, PT_NORMAL, NULL },
+	{ PROP_PASSWORD_SITES_REMOVE, "password_sites_remove_button", NULL, PT_NORMAL, NULL },
 
 	{ -1, NULL, NULL }
 };
@@ -160,17 +201,16 @@ cookies_treeview_selection_changed_cb (GtkTreeSelection *selection,
 	GtkWidget *widget;
 	GList *l;
 	GaleonDialog *d = GALEON_DIALOG(dialog);
-	gboolean has_selection;
+	gboolean can_properties;
 	
 	l = gul_gui_treeview_get_selection_refs 
 		(GTK_TREE_VIEW(dialog->priv->cookies->treeview));
 
-	has_selection = l != NULL;
+	can_properties = (l != NULL && l->next == NULL);
+	gul_gui_treeview_free_selection_refs (l);
 	
 	widget = galeon_dialog_get_control (d, PROP_COOKIES_PROPERTIES);
-	gtk_widget_set_sensitive (widget, has_selection);
-
-	g_list_free (l);
+	gtk_widget_set_sensitive (widget, can_properties);
 }
 
 static void
@@ -186,11 +226,10 @@ action_treeview_selection_changed_cb (GtkTreeSelection *selection,
 		(GTK_TREE_VIEW(action->treeview));
 
 	has_selection = l != NULL;
+	gul_gui_treeview_free_selection_refs (l);
 
 	widget = galeon_dialog_get_control (d, action->remove_id);
 	gtk_widget_set_sensitive (widget, has_selection);
-	
-	g_list_free (l);
 }
 
 static GtkWidget *
@@ -247,6 +286,60 @@ setup_passwords_treeview (PdmDialog *dialog)
 	return GTK_WIDGET (treeview);
 }
 
+static GtkWidget *
+setup_password_sites_treeview (PdmDialog *dialog)
+{
+
+	GtkTreeView *treeview;
+        GtkListStore *liststore;
+        GtkCellRenderer *renderer;
+        GtkTreeViewColumn *column;
+	GtkTreeSelection *selection;
+
+	treeview = GTK_TREE_VIEW(galeon_dialog_get_control 
+				 (GALEON_DIALOG(dialog),
+				 PROP_PASSWORD_SITES_TREEVIEW));
+	
+        /* set tree model */
+        liststore = gtk_list_store_new(3,
+                                       G_TYPE_STRING,
+                                       G_TYPE_STRING,
+                                       G_TYPE_POINTER);
+        gtk_tree_view_set_model(treeview, GTK_TREE_MODEL(liststore));
+        gtk_tree_view_set_headers_visible(treeview, TRUE);
+	selection = gtk_tree_view_get_selection(treeview);
+	gtk_tree_selection_set_mode(selection,
+				    GTK_SELECTION_MULTIPLE);
+	
+        renderer = gtk_cell_renderer_text_new();
+
+        gtk_tree_view_insert_column_with_attributes (treeview,
+                                                     COL_PASSWORD_SITES_HOST, 
+						     _("Host"),
+                                                     renderer,
+                                                     "text", COL_PASSWORD_SITES_HOST,
+                                                     NULL);
+        column = gtk_tree_view_get_column(treeview, COL_PASSWORD_SITES_HOST);
+        gtk_tree_view_column_set_resizable(column, TRUE);
+        gtk_tree_view_column_set_reorderable(column, TRUE);
+        gtk_tree_view_column_set_sizing(column, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
+        gtk_tree_view_column_set_sort_column_id(column, COL_PASSWORD_SITES_HOST);
+
+	gtk_tree_view_insert_column_with_attributes(treeview,
+                                                    COL_PASSWORD_SITES_STATUS, 
+						    _("Status"),
+                                                    renderer,
+                                                    "text", COL_PASSWORD_SITES_STATUS,
+                                                    NULL);
+        column = gtk_tree_view_get_column(treeview, COL_PASSWORD_SITES_STATUS);
+        gtk_tree_view_column_set_resizable(column, TRUE);
+        gtk_tree_view_column_set_reorderable(column, TRUE);
+        gtk_tree_view_column_set_sizing(column, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
+        gtk_tree_view_column_set_sort_column_id(column, COL_PASSWORD_SITES_STATUS);
+
+	return GTK_WIDGET(treeview);
+}
+
 static GtkWidget *
 setup_cookies_treeview (PdmDialog *dialog)
 {
@@ -306,25 +399,135 @@ setup_cookies_treeview (PdmDialog *dialog)
 	return GTK_WIDGET(treeview);
 }
 
+static GtkWidget *
+setup_cookie_sites_treeview (PdmDialog *dialog)
+{
+	GtkTreeView *treeview;
+        GtkListStore *liststore;
+        GtkCellRenderer *renderer;
+        GtkTreeViewColumn *column;
+	GtkTreeSelection *selection;
+	
+	treeview = GTK_TREE_VIEW (galeon_dialog_get_control 
+				  (GALEON_DIALOG(dialog),
+				  PROP_COOKIE_SITES_TREEVIEW));
+
+	
+	
+        /* set tree model */
+        liststore = gtk_list_store_new (3,
+                                        G_TYPE_STRING,
+                                        G_TYPE_STRING,
+                                        G_TYPE_POINTER);
+        gtk_tree_view_set_model (treeview, GTK_TREE_MODEL(liststore));
+        gtk_tree_view_set_headers_visible (treeview, TRUE);
+	selection = gtk_tree_view_get_selection (treeview);
+	gtk_tree_selection_set_mode (selection,
+				     GTK_SELECTION_MULTIPLE);
+
+	renderer = gtk_cell_renderer_text_new ();
+
+	gtk_tree_view_insert_column_with_attributes (treeview,
+                                                     COL_COOKIE_SITES_DOMAIN, 
+						     _("Domain"),
+                                                     renderer,
+                                                     "text", COL_COOKIE_SITES_DOMAIN,
+                                                     NULL);
+        column = gtk_tree_view_get_column (treeview, COL_COOKIE_SITES_DOMAIN);
+        gtk_tree_view_column_set_resizable (column, TRUE);
+        gtk_tree_view_column_set_reorderable (column, TRUE);
+        gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
+        gtk_tree_view_column_set_sort_column_id (column, COL_COOKIE_SITES_DOMAIN);
+	
+        gtk_tree_view_insert_column_with_attributes (treeview,
+                                                     COL_COOKIE_SITES_STATUS, 
+						     _("Status"),
+                                                     renderer,
+                                                     "text", COL_COOKIE_SITES_STATUS,
+                                                     NULL);
+        column = gtk_tree_view_get_column (treeview, COL_COOKIE_SITES_STATUS);
+        gtk_tree_view_column_set_resizable (column, TRUE);
+        gtk_tree_view_column_set_reorderable (column, TRUE);
+        gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
+        gtk_tree_view_column_set_sort_column_id (column, COL_COOKIE_SITES_STATUS);
+
+	return GTK_WIDGET(treeview);
+}
+
+static GtkWidget *
+setup_image_sites_treeview (PdmDialog *dialog)
+{
+	GtkTreeView *treeview;
+        GtkListStore *liststore;
+        GtkCellRenderer *renderer;
+        GtkTreeViewColumn *column;
+	GtkTreeSelection *selection;
+	
+	treeview = GTK_TREE_VIEW(galeon_dialog_get_control 
+				 (GALEON_DIALOG(dialog),
+				 PROP_IMAGE_SITES_TREEVIEW));
+
+	
+	
+        /* set tree model */
+        liststore = gtk_list_store_new(3,
+                                       G_TYPE_STRING,
+                                       G_TYPE_STRING,
+                                       G_TYPE_POINTER);
+        gtk_tree_view_set_model(treeview, GTK_TREE_MODEL(liststore));
+        gtk_tree_view_set_headers_visible(treeview, TRUE);
+	selection = gtk_tree_view_get_selection(treeview);
+	gtk_tree_selection_set_mode(selection,
+				    GTK_SELECTION_MULTIPLE);
+
+	renderer = gtk_cell_renderer_text_new();
+
+	gtk_tree_view_insert_column_with_attributes(treeview,
+                                                    COL_IMAGE_SITES_DOMAIN, 
+						    _("Domain"),
+                                                    renderer,
+                                                    "text", COL_IMAGE_SITES_DOMAIN,
+                                                    NULL);
+        column = gtk_tree_view_get_column(treeview, COL_IMAGE_SITES_DOMAIN);
+        gtk_tree_view_column_set_resizable(column, TRUE);
+        gtk_tree_view_column_set_reorderable(column, TRUE);
+        gtk_tree_view_column_set_sizing(column, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
+        gtk_tree_view_column_set_sort_column_id(column, COL_IMAGE_SITES_DOMAIN);
+	
+        gtk_tree_view_insert_column_with_attributes(treeview,
+                                                    COL_IMAGE_SITES_STATUS, 
+						    _("Status"),
+                                                    renderer,
+                                                    "text", COL_IMAGE_SITES_STATUS,
+                                                    NULL);
+        column = gtk_tree_view_get_column(treeview, COL_IMAGE_SITES_STATUS);
+        gtk_tree_view_column_set_resizable(column, TRUE);
+        gtk_tree_view_column_set_reorderable(column, TRUE);
+        gtk_tree_view_column_set_sizing(column, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
+        gtk_tree_view_column_set_sort_column_id(column, COL_IMAGE_SITES_STATUS);
+
+	return GTK_WIDGET(treeview);
+}
+
 static void
 pdm_dialog_remove_button_clicked_cb (GtkWidget *button,
 				     PdmActionInfo *action)
 {
-	GList *l;
+	GList *refs, *l;
 	GList *remove_list = NULL;
 	GtkTreeModel *model;
-	GValue val = {0, };
 
 	model = gtk_tree_view_get_model 
 		(GTK_TREE_VIEW(action->treeview));
-	l = gul_gui_treeview_get_selection_refs 
+	refs = gul_gui_treeview_get_selection_refs 
 		(GTK_TREE_VIEW(action->treeview));
 
-	for (; l != NULL; l = l->next)
+	for (l = refs; l != NULL; l = l->next)
 	{
 		GtkTreeIter iter;
 		gpointer data;
 		GtkTreePath *path;
+		GValue val = {0, };
 
 		path = gtk_tree_row_reference_get_path 
 			((GtkTreeRowReference *)l->data); 
@@ -341,6 +544,8 @@ pdm_dialog_remove_button_clicked_cb (GtkWidget *button,
 		remove_list = g_list_append (remove_list, data);
 	}
 
+	gul_gui_treeview_free_selection_refs (refs);
+
 	if (remove_list)
 	{
 		action->remove (action, remove_list);
@@ -348,22 +553,6 @@ pdm_dialog_remove_button_clicked_cb (GtkWidget *button,
 	}
 }
 
-static void
-pdm_dialog_clear_button_clicked_cb (GtkWidget *button,
-				    PdmActionInfo *action)
-{
-	GtkListStore *store;
-
-	store = GTK_LIST_STORE(gtk_tree_view_get_model 
-			       (GTK_TREE_VIEW(action->treeview)));
-	gtk_list_store_clear (store);
-
-	action->remove (action, action->list);
-
-	action->free (action, NULL);
-	action->list = NULL;
-}
-
 static void
 setup_action (PdmActionInfo *action)
 {
@@ -382,12 +571,6 @@ setup_action (PdmActionInfo *action)
 			  G_CALLBACK(pdm_dialog_remove_button_clicked_cb),
 			  action);
 	
-	widget = galeon_dialog_get_control (GALEON_DIALOG(action->dialog), 
-					    action->clear_id);
-	g_signal_connect (widget, "clicked",
-			  G_CALLBACK(pdm_dialog_clear_button_clicked_cb),
-			  action);
-
 	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW(action->treeview));
 	g_signal_connect (selection, "changed",
 			  G_CALLBACK(action_treeview_selection_changed_cb),
@@ -408,7 +591,7 @@ pdm_dialog_cookie_add (PdmActionInfo *info,
 	gtk_list_store_append (store, &iter);
         gtk_list_store_set (store,
                             &iter,
-			    COL_COOKIES_HOST, cinfo->base.domain,
+			    COL_COOKIES_HOST, cinfo->domain,
                             COL_COOKIES_NAME, cinfo->name,
 			    COL_COOKIES_DATA, cinfo,
                             -1);
@@ -434,6 +617,70 @@ pdm_dialog_password_add (PdmActionInfo *info,
                             -1);
 }
 
+static void
+pdm_dialog_password_site_add(PdmActionInfo *info, 
+			     gpointer password_site)
+{
+	GtkListStore *store;
+	GtkTreeIter iter;
+	PasswordInfo *pinfo = (PasswordInfo *)password_site;
+
+	store = GTK_LIST_STORE(gtk_tree_view_get_model 
+			       (GTK_TREE_VIEW(info->treeview)));
+
+	gtk_list_store_append(store, &iter);
+        gtk_list_store_set(store,
+                           &iter,
+                           COL_PASSWORD_SITES_HOST, pinfo->host,
+                           COL_PASSWORD_SITES_STATUS, _("Cannot save passwords"),
+			   COL_PASSWORD_SITES_DATA, pinfo,
+                           -1);
+}
+
+static void
+pdm_dialog_cookie_site_add (PdmActionInfo *info, 
+		            gpointer cookie_site)
+{
+	GtkListStore *store;
+	GtkTreeIter iter;
+	PermissionInfo *pinfo = (PermissionInfo *)cookie_site;
+	
+	store = GTK_LIST_STORE(gtk_tree_view_get_model 
+			       (GTK_TREE_VIEW(info->treeview)));
+
+	gtk_list_store_append (store, &iter);
+        gtk_list_store_set (store,
+                            &iter,
+			    COL_COOKIE_SITES_DOMAIN, pinfo->domain,
+                            COL_COOKIE_SITES_STATUS, pinfo->status 
+			                             ? _("Can set cookies") 
+						     : _("Cannot set cookies"),
+			    COL_COOKIE_SITES_DATA,   pinfo,
+                            -1);
+}
+
+static void
+pdm_dialog_image_site_add (PdmActionInfo *info, 
+		            gpointer image_site)
+{
+	GtkListStore *store;
+	GtkTreeIter iter;
+	PermissionInfo *pinfo = (PermissionInfo *)image_site;
+	
+	store = GTK_LIST_STORE(gtk_tree_view_get_model 
+			       (GTK_TREE_VIEW(info->treeview)));
+
+	gtk_list_store_append(store, &iter);
+        gtk_list_store_set(store,
+                           &iter,
+			   COL_IMAGE_SITES_DOMAIN, pinfo->domain,
+                           COL_IMAGE_SITES_STATUS, pinfo->status 
+			                           ? _("Can show images") 
+						   : _("Cannot show images"),
+			   COL_IMAGE_SITES_DATA,   pinfo,
+                           -1);
+}
+
 static void
 pdm_dialog_cookie_remove (PdmActionInfo *info,
 			  GList *data)
@@ -452,6 +699,33 @@ pdm_dialog_password_remove (PdmActionInfo *info,
 	galeon_embed_shell_remove_passwords (shell, data, PASSWORD_PASSWORD);
 }
 
+static void
+pdm_dialog_password_site_remove(PdmActionInfo *info,
+				GList *data)
+{
+	GaleonEmbedShell *shell;
+	shell = galeon_shell_get_embed_shell(galeon_shell);
+	galeon_embed_shell_remove_passwords(shell, data, PASSWORD_REJECT);
+}
+
+static void
+pdm_dialog_cookie_site_remove (PdmActionInfo *info,
+			       GList *data)
+{
+	GaleonEmbedShell *shell;
+	shell = galeon_shell_get_embed_shell (galeon_shell);
+	galeon_embed_shell_remove_permissions (shell, COOKIES_PERMISSION, data);
+}
+
+static void
+pdm_dialog_image_site_remove(PdmActionInfo *info,
+			     GList *data)
+{
+	GaleonEmbedShell *shell;
+	shell = galeon_shell_get_embed_shell(galeon_shell);
+	galeon_embed_shell_remove_permissions(shell, IMAGES_PERMISSION, data);
+}
+
 static void
 pdm_dialog_cookies_free (PdmActionInfo *info,
 			 GList *data)
@@ -476,14 +750,60 @@ pdm_dialog_passwords_free (PdmActionInfo *info,
 	galeon_embed_shell_free_passwords (shell, l);
 }
 
+static void
+pdm_dialog_site_permissions_free(PdmActionInfo *info,
+				 GList *data)
+{
+	GaleonEmbedShell *shell;
+	GList *l;
+	
+	shell = galeon_shell_get_embed_shell (galeon_shell);
+	l = data ? data : info->list;
+	galeon_embed_shell_free_permissions (shell, l);
+}
+
+/* Group all Properties and Remove buttons in the same size group to avoid the
+ * little jerk you get otherwise when switching pages because one set of
+ * buttons is wider than another. */
+static void
+group_button_allocations (GaleonDialog *dialog)
+{
+	const gint props[] =
+	{
+		PROP_COOKIES_REMOVE,
+		PROP_COOKIES_PROPERTIES,
+		PROP_PASSWORDS_REMOVE,
+		PROP_COOKIE_SITES_REMOVE,
+		PROP_IMAGE_SITES_REMOVE,
+		PROP_PASSWORD_SITES_REMOVE,
+	};
+	GtkSizeGroup *size_group;
+	guint i;
+
+	size_group = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
+
+	for (i = 0; i < G_N_ELEMENTS (props); ++i)
+	{
+		GtkWidget *w;
+		w = galeon_dialog_get_control (dialog, props[i]);
+		gtk_size_group_add_widget (size_group,  w);
+	}
+}
+
 static void
 pdm_dialog_init (PdmDialog *dialog)
 {	
 	GaleonEmbedShell *shell;
 	PdmActionInfo *cookies;
 	PdmActionInfo *passwords;
+	PdmActionInfo *cookie_sites;
+	PdmActionInfo *image_sites;
+	PdmActionInfo *password_sites;
 	GtkWidget *cookies_tv;
 	GtkWidget *passwords_tv;
+	GtkWidget *cookie_sites_tv;
+	GtkWidget *image_sites_tv;
+	GtkWidget *password_sites_tv;
 	
 	shell = galeon_shell_get_embed_shell (galeon_shell);
 	
@@ -496,14 +816,18 @@ pdm_dialog_init (PdmDialog *dialog)
                                  "galeon.glade",
                                  "pdm_dialog");
 
+	group_button_allocations (GALEON_DIALOG (dialog));
+
 	cookies_tv = setup_cookies_treeview (dialog);
 	passwords_tv = setup_passwords_treeview (dialog);
+	cookie_sites_tv = setup_cookie_sites_treeview (dialog);
+	image_sites_tv = setup_image_sites_treeview(dialog);
+	password_sites_tv = setup_password_sites_treeview(dialog);
 
 	cookies = g_new0 (PdmActionInfo, 1);
 	galeon_embed_shell_list_cookies (shell, &cookies->list);
 	cookies->dialog = dialog;
 	cookies->remove_id = PROP_COOKIES_REMOVE;
-	cookies->clear_id = PROP_COOKIES_CLEAR;
 	cookies->add = pdm_dialog_cookie_add;
 	cookies->remove = pdm_dialog_cookie_remove;
 	cookies->free = pdm_dialog_cookies_free;
@@ -516,7 +840,6 @@ pdm_dialog_init (PdmDialog *dialog)
 					   &passwords->list);
 	passwords->dialog = dialog;
 	passwords->remove_id = PROP_PASSWORDS_REMOVE;
-	passwords->clear_id = PROP_PASSWORDS_CLEAR;
 	passwords->add = pdm_dialog_password_add;
 	passwords->remove = pdm_dialog_password_remove;
 	passwords->free = pdm_dialog_passwords_free;
@@ -524,8 +847,47 @@ pdm_dialog_init (PdmDialog *dialog)
 	passwords->data_col = COL_PASSWORDS_DATA;
 	setup_action (passwords);
 
+	password_sites = g_new0(PdmActionInfo, 1);
+	galeon_embed_shell_list_passwords(shell, PASSWORD_REJECT,
+					  &password_sites->list);
+	password_sites->dialog = dialog;
+	password_sites->remove_id = PROP_PASSWORD_SITES_REMOVE;
+	password_sites->add = pdm_dialog_password_site_add;
+	password_sites->remove = pdm_dialog_password_site_remove;
+	password_sites->free = pdm_dialog_passwords_free;
+	password_sites->treeview = password_sites_tv;
+	password_sites->data_col = COL_PASSWORD_SITES_DATA;
+	setup_action(password_sites);
+
+	cookie_sites = g_new0 (PdmActionInfo, 1);
+	galeon_embed_shell_list_permissions (shell, COOKIES_PERMISSION,
+					     &cookie_sites->list);
+	cookie_sites->dialog	= dialog;
+	cookie_sites->remove_id = PROP_COOKIE_SITES_REMOVE;
+	cookie_sites->add	= pdm_dialog_cookie_site_add;
+	cookie_sites->remove	= pdm_dialog_cookie_site_remove;
+	cookie_sites->free	= pdm_dialog_site_permissions_free;
+	cookie_sites->treeview	= cookie_sites_tv;
+	cookie_sites->data_col	= COL_COOKIE_SITES_DATA;
+	setup_action (cookie_sites);
+
+	image_sites = g_new0(PdmActionInfo, 1);
+	galeon_embed_shell_list_permissions(shell, IMAGES_PERMISSION,
+					    &image_sites->list);
+	image_sites->dialog	= dialog;
+	image_sites->remove_id	= PROP_IMAGE_SITES_REMOVE;
+	image_sites->add	= pdm_dialog_image_site_add;
+	image_sites->remove	= pdm_dialog_image_site_remove;
+	image_sites->free	= pdm_dialog_site_permissions_free;
+	image_sites->treeview	= image_sites_tv;
+	image_sites->data_col	= COL_IMAGE_SITES_DATA;
+	setup_action(image_sites);
+
 	dialog->priv->cookies = cookies;
 	dialog->priv->passwords = passwords;
+	dialog->priv->password_sites = password_sites;
+	dialog->priv->cookie_sites = cookie_sites;
+	dialog->priv->image_sites = image_sites;
 }
 
 static void
@@ -540,7 +902,10 @@ pdm_dialog_finalize (GObject *object)
 
         g_return_if_fail (dialog->priv != NULL);
 
+	pdm_dialog_site_permissions_free (dialog->priv->cookie_sites, NULL);
+	pdm_dialog_site_permissions_free (dialog->priv->image_sites, NULL);
 	pdm_dialog_passwords_free (dialog->priv->passwords, NULL);
+	pdm_dialog_passwords_free (dialog->priv->password_sites, NULL);
 	pdm_dialog_cookies_free (dialog->priv->cookies, NULL);
 	
         g_free (dialog->priv);
@@ -562,11 +927,10 @@ pdm_dialog_new (GtkWidget *window)
 
 static void
 show_cookies_properties (PdmDialog *dialog,
-			 CookieInfo *info)
+			 const CookieInfo *info)
 {
-	GtkWidget *gdialog;
 	GtkWidget *table;
-	GtkWidget *label;
+	GtkWidget *gdialog;
 	GtkWidget *parent;
 	GtkWidget *dialog_vbox;
 
@@ -581,60 +945,7 @@ show_cookies_properties (PdmDialog *dialog,
 	gtk_dialog_set_has_separator (GTK_DIALOG(gdialog), FALSE);
 	gtk_container_set_border_width (GTK_CONTAINER(gdialog), 6);
 
-	table = gtk_table_new (2, 4, FALSE);
-	gtk_container_set_border_width (GTK_CONTAINER(table), 6);
-	gtk_table_set_row_spacings (GTK_TABLE(table), 10);
-	gtk_table_set_col_spacings (GTK_TABLE(table), 10);
-	gtk_widget_show (table);
-	
-	label = gtk_label_new (_("<b>Value</b>"));
-	gtk_label_set_use_markup (GTK_LABEL(label), TRUE);
-	gtk_misc_set_alignment (GTK_MISC(label), 0, 0);
-	gtk_widget_show (label);
-	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 0, 1,
-			  GTK_FILL, GTK_FILL, 0, 0);
-	
-	label = gul_ellipsizing_label_new (info->value);
-	gtk_misc_set_alignment (GTK_MISC(label), 0, 0);
-	gtk_widget_show (label);
-	gtk_table_attach_defaults (GTK_TABLE (table), label, 1, 2, 0, 1);
-
-	label = gtk_label_new (_("<b>Path</b>"));
-	gtk_label_set_use_markup (GTK_LABEL(label), TRUE);
-	gtk_misc_set_alignment (GTK_MISC(label), 0, 0);
-	gtk_widget_show (label);
-	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 1, 2,
-			  GTK_FILL, GTK_FILL, 0, 0);
-	
-	label = gtk_label_new (info->path);
-	gtk_misc_set_alignment (GTK_MISC(label), 0, 0);
-	gtk_widget_show (label);
-	gtk_table_attach_defaults (GTK_TABLE (table), label, 1, 2, 1, 2);
-
-	label = gtk_label_new (_("<b>Secure</b>"));
-	gtk_label_set_use_markup (GTK_LABEL(label), TRUE);
-	gtk_misc_set_alignment (GTK_MISC(label), 0, 0);
-	gtk_widget_show (label);
-	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 2, 3,
-			  GTK_FILL, GTK_FILL, 0, 0);
-	
-	label = gtk_label_new (info->secure);
-	gtk_misc_set_alignment (GTK_MISC(label), 0, 0);
-	gtk_widget_show (label);
-	gtk_table_attach_defaults (GTK_TABLE (table), label, 1, 2, 2, 3);
-
-	label = gtk_label_new (_("<b>Expire</b>"));
-	gtk_label_set_use_markup (GTK_LABEL(label), TRUE);
-	gtk_misc_set_alignment (GTK_MISC(label), 0, 0);
-	gtk_widget_show (label);
-	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 3, 4,
-			  GTK_FILL, GTK_FILL, 0, 0);
-	
-	label = gtk_label_new (info->expire);
-	gtk_misc_set_alignment (GTK_MISC(label), 0, 0.5);
-	gtk_widget_show (label);
-	gtk_table_attach_defaults (GTK_TABLE (table), label, 1, 2, 3, 4);
-	
+	table = cookie_info_table_new (info);
 	dialog_vbox = GTK_DIALOG(gdialog)->vbox;
 	gtk_box_pack_start (GTK_BOX(dialog_vbox), 
                             table,
@@ -664,6 +975,8 @@ pdm_dialog_cookies_properties_button_clicked_cb (GtkWidget *button,
 
 	path = gtk_tree_row_reference_get_path 
 		((GtkTreeRowReference *)l->data); 
+	gul_gui_treeview_free_selection_refs (l);
+
         gtk_tree_model_get_iter (model, &iter, path);
 	gtk_tree_model_get_value 
 		(model, &iter, COL_COOKIES_DATA, &val);
