@@ -72,7 +72,7 @@ void	      session_save     		(Session *session,
 
 gboolean      session_autoresume   	(Session *session);
 
-GList        *session_get_windows     	(Session *session);
+const GList  *session_get_windows     	(Session *session);
 
 void          session_add_window      	(Session *session,
 					 GaleonWindow *window);
