@@ -500,7 +500,52 @@ galeon_marshal_VOID__INT_INT (GClosure     *closure,
             data2);
 }
 
-/* INT:OBJECT (./galeon-marshal.list:14) */
+/* VOID:INT,INT,INT (./galeon-marshal.list:14) */
+extern void galeon_marshal_VOID__INT_INT_INT (GClosure     *closure,
+                                              GValue       *return_value,
+                                              guint         n_param_values,
+                                              const GValue *param_values,
+                                              gpointer      invocation_hint,
+                                              gpointer      marshal_data);
+void
+galeon_marshal_VOID__INT_INT_INT (GClosure     *closure,
+                                  GValue       *return_value,
+                                  guint         n_param_values,
+                                  const GValue *param_values,
+                                  gpointer      invocation_hint,
+                                  gpointer      marshal_data)
+{
+  typedef void (*GMarshalFunc_VOID__INT_INT_INT) (gpointer     data1,
+                                                  gint         arg_1,
+                                                  gint         arg_2,
+                                                  gint         arg_3,
+                                                  gpointer     data2);
+  register GMarshalFunc_VOID__INT_INT_INT callback;
+  register GCClosure *cc = (GCClosure*) closure;
+  register gpointer data1, data2;
+
+  g_return_if_fail (n_param_values == 4);
+
+  if (G_CCLOSURE_SWAP_DATA (closure))
+    {
+      data1 = closure->data;
+      data2 = g_value_peek_pointer (param_values + 0);
+    }
+  else
+    {
+      data1 = g_value_peek_pointer (param_values + 0);
+      data2 = closure->data;
+    }
+  callback = (GMarshalFunc_VOID__INT_INT_INT) (marshal_data ? marshal_data : cc->callback);
+
+  callback (data1,
+            g_marshal_value_peek_int (param_values + 1),
+            g_marshal_value_peek_int (param_values + 2),
+            g_marshal_value_peek_int (param_values + 3),
+            data2);
+}
+
+/* INT:OBJECT (./galeon-marshal.list:15) */
 extern void galeon_marshal_INT__OBJECT (GClosure     *closure,
                                         GValue       *return_value,
                                         guint         n_param_values,
