@@ -24,27 +24,26 @@
  */
 
 #include "gul-notebook.h"
-#include "eel-gconf-extensions.h"
-#include "prefs-strings.h"
+#include "galeon-marshal.h"
 
 #include <gtk/gtk.h>
 #include <glib-object.h>
 #include <libgnome/gnome-i18n.h>
 
 #define AFTER_ALL_TABS -1
-
+#define NOT_IN_APP_WINDOWS -2
 #define TAB_MIN_SIZE 60
-#define TAB_MAX_SIZE 8
+#define TAB_NB_MAX    8
 
 struct GulNotebookPrivate 
 {
-	GtkStyle *loading_text_style;
-	GtkStyle *new_text_style;
-	gboolean colors_loaded;
 	GList *focused_pages;
+	GList *opened_tabs;
+	GtkPolicyType policy;
 	
 	/* Used during tab drag'n'drop */
 	gulong motion_notify_handler_id;
+	gint x_start, y_start;
 	gboolean drag_in_progress;
 	GulNotebook *src_notebook;
 	gint src_page;
@@ -57,6 +56,28 @@ static void gul_notebook_finalize     (GObject *object);
 
 /* Local variables */
 static GdkCursor *cursor = NULL;
+static GList *notebooks  = NULL;
+
+
+/* Local functions */
+static void drag_start (GulNotebook *notebook, 
+			GulNotebook *src_notebook,
+			gint src_page);
+static void drag_stop  (GulNotebook *notebook);
+
+static gboolean motion_notify_cb (GulNotebook *notebook, 
+				  GdkEventMotion *event, 
+				  gpointer data);
+
+/* Signals */
+enum
+{
+	TAB_DROPPED,
+	TAB_DETACHED,
+	LAST_SIGNAL
+};
+
+static guint gul_notebook_signals[LAST_SIGNAL] = { 0 };
 
 GType 
 gul_notebook_get_type (void)
@@ -90,8 +111,36 @@ static void
 gul_notebook_class_init (GulNotebookClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
-
 	object_class->finalize = gul_notebook_finalize;
+
+	/* init signals */
+	gul_notebook_signals[TAB_DROPPED] =
+		g_signal_new ("tab_dropped",
+			      G_OBJECT_CLASS_TYPE (object_class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (GulNotebookClass, 
+					       tab_dropped),
+			      NULL, NULL,
+			      galeon_marshal_VOID__OBJECT_OBJECT_INT,
+			      G_TYPE_NONE,
+			      3, 
+			      GTK_TYPE_WIDGET,
+			      GUL_NOTEBOOK_TYPE,
+			      G_TYPE_INT);
+	gul_notebook_signals[TAB_DETACHED] =
+		g_signal_new ("tab_detached",
+			      G_OBJECT_CLASS_TYPE (object_class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (GulNotebookClass, 
+					       tab_detached),
+			      NULL, NULL,
+			      galeon_marshal_VOID__INT_INT_INT,
+			      G_TYPE_NONE,
+			      3, 
+			      G_TYPE_INT,
+			      G_TYPE_INT,
+			      G_TYPE_INT);
+
 }
 
 static gboolean 
@@ -107,7 +156,6 @@ is_in_notebook_window (GulNotebook *notebook,
 	gdk_window_get_origin (window, &x, &y);
 	rel_x = abs_x - x;
 	rel_y = abs_y - y;
-	g_assert (rel_x>=0 && rel_y>=0);
 
 	x = GTK_WIDGET(notebook)->allocation.x;
 	y = GTK_WIDGET(notebook)->allocation.y;
@@ -116,17 +164,54 @@ is_in_notebook_window (GulNotebook *notebook,
 	return ((rel_x>=x) && (rel_y>=y) && (rel_x<=x+width) && (rel_y<=y+height));
 }
 
+static GulNotebook *
+find_notebook_at_pointer (gint abs_x, gint abs_y)
+{
+	GList *l;
+	gint x, y;
+	GdkWindow *win_at_pointer = gdk_window_at_pointer (&x, &y);
+	GdkWindow *parent_at_pointer = NULL;
+
+	if (win_at_pointer == NULL) 
+	{
+		/* We are outside all windows containing a notebook */
+		return NULL;
+	}
+
+	gdk_window_get_toplevel (win_at_pointer);
+	/* When we are in the notebook event window, win_at_pointer will be
+	   this event window, and the toplevel window we are interested in
+	   will be its parent
+	*/
+	parent_at_pointer = gdk_window_get_parent (win_at_pointer);
+
+	for (l = notebooks; l != NULL; l = l->next) 
+	{
+		GulNotebook *nb = GUL_NOTEBOOK (l->data);
+		GdkWindow *win = GTK_WIDGET (nb)->window;
+
+		win = gdk_window_get_toplevel (win);
+		if (((win == win_at_pointer) || (win == parent_at_pointer))
+		    && is_in_notebook_window (nb, abs_x, abs_y)) 
+		{
+			return nb;
+		}
+	}
+	return NULL;
+}
+
+
 static gint
 find_tab_num_at_pos (GulNotebook *notebook, gint abs_x, gint abs_y)
 {
 	GtkPositionType tab_pos;
 	int page_num = 0;
-	GtkNotebook *nb = GTK_NOTEBOOK(notebook);
+	GtkNotebook *nb = GTK_NOTEBOOK (notebook);
 	GtkWidget *page;
 	
-	tab_pos = gtk_notebook_get_tab_pos (GTK_NOTEBOOK(notebook));
+	tab_pos = gtk_notebook_get_tab_pos (GTK_NOTEBOOK (notebook));
 
-	if (GTK_NOTEBOOK(notebook)->first_tab == NULL) 
+	if (GTK_NOTEBOOK (notebook)->first_tab == NULL) 
 	{
 		return AFTER_ALL_TABS;
 	}
@@ -140,15 +225,15 @@ find_tab_num_at_pos (GulNotebook *notebook, gint abs_x, gint abs_y)
 		gint x_root, y_root;
 		
 		tab = gtk_notebook_get_tab_label (nb, page);
-		g_return_val_if_fail(tab != NULL, -1);
+		g_return_val_if_fail (tab != NULL, -1);
 		
-		if (!GTK_WIDGET_MAPPED(GTK_WIDGET(tab))) 
+		if (!GTK_WIDGET_MAPPED (GTK_WIDGET (tab))) 
 		{
 			page_num++;
 			continue;
 		}
 
-		gdk_window_get_origin (GDK_WINDOW(tab->window), 
+		gdk_window_get_origin (GDK_WINDOW (tab->window), 
 				       &x_root, &y_root);
 
 		max_x = x_root + tab->allocation.x + tab->allocation.width;
@@ -159,9 +244,10 @@ find_tab_num_at_pos (GulNotebook *notebook, gint abs_x, gint abs_y)
 		    &&(abs_x<=max_x)) 
 		{
 			return page_num;
-		} else if (((tab_pos == GTK_POS_LEFT) 
-			    || (tab_pos == GTK_POS_RIGHT))
-			   && (abs_y<=max_y)) 
+		} 
+		else if (((tab_pos == GTK_POS_LEFT) 
+			  || (tab_pos == GTK_POS_RIGHT))
+			 && (abs_y<=max_y)) 
 		{
 			return page_num;
 		}		      
@@ -171,295 +257,34 @@ find_tab_num_at_pos (GulNotebook *notebook, gint abs_x, gint abs_y)
 	return AFTER_ALL_TABS;
 }
 
-static gboolean
-is_notebook_at_pointer (GulNotebook *nb,
-			gint abs_x, gint abs_y)
+
+static gint find_notebook_and_tab_at_pos (gint abs_x, gint abs_y, 
+					  GulNotebook **notebook, 
+					  gint *page_num)
 {
-	gint x, y;
-	GdkWindow *win_at_pointer = gdk_window_at_pointer (&x, &y);
-	GdkWindow *parent_at_pointer = NULL;
-	GdkWindow *win;
+	*notebook = find_notebook_at_pointer (abs_x, abs_y);
+	if (*notebook == NULL) 
+	{
+		return NOT_IN_APP_WINDOWS;
+	}
+	*page_num = find_tab_num_at_pos (*notebook, abs_x, abs_y);
 	
-	if (win_at_pointer == NULL) 
+	if (*page_num < 0) 
 	{
-		return FALSE;
+		return *page_num;
 	}
-
-	gdk_window_get_toplevel(win_at_pointer);
-	/* When we are in the notebook event window, win_at_pointer will be
-	   this event window, and the toplevel window we are interested in
-	   will be its parent
-	*/
-	parent_at_pointer = gdk_window_get_parent (win_at_pointer);
-
-	win = GTK_WIDGET(nb)->window;
-	win = gdk_window_get_toplevel (win);
-
-	return ((win == win_at_pointer || win == parent_at_pointer)
-	        && is_in_notebook_window (nb, abs_x, abs_y));
-}
-
-static void 
-move_tab (GulNotebook *notebook, gint dest_page_num)
-{
-	gint cur_page_num;
-
-	cur_page_num = gtk_notebook_get_current_page (GTK_NOTEBOOK(notebook));
-
-	if (dest_page_num != cur_page_num)
+	else 
 	{
-		GtkWidget *cur_page = 
-			gtk_notebook_get_nth_page (GTK_NOTEBOOK(notebook),
-						   cur_page_num);
-		gtk_notebook_reorder_child (GTK_NOTEBOOK(notebook), cur_page,
-					    dest_page_num);
+		return 0;
 	}
 }
 
-static void 
-drag_start (GulNotebook *notebook, 
-	    GulNotebook *src_notebook, 
-	    gint src_page)
-{
-	notebook->priv->drag_in_progress = TRUE;
-	notebook->priv->src_notebook = src_notebook;
-	notebook->priv->src_page = src_page;
-
-	/* get a new cursor, if necessary */
-	if (!cursor) cursor = gdk_cursor_new (GDK_FLEUR);
-	
-	/* grab the pointer */
-	gtk_grab_add (GTK_WIDGET(notebook));
-	if (!gdk_pointer_is_grabbed ()) {
-		gdk_pointer_grab (GDK_WINDOW(GTK_WIDGET(notebook)->window), 
-				  FALSE,
-				  GDK_BUTTON1_MOTION_MASK | GDK_BUTTON_RELEASE_MASK,
-				  NULL, cursor, GDK_CURRENT_TIME);
-	}
-}
-
-static void 
-drag_stop (GulNotebook *notebook)
-{
-	notebook->priv->drag_in_progress = FALSE;
-	notebook->priv->src_notebook = NULL;
-	notebook->priv->src_page = -1;
-	if (notebook->priv->motion_notify_handler_id != 0) 
-	{
-		g_signal_handler_disconnect (G_OBJECT(notebook), 
-					     notebook->priv->motion_notify_handler_id);
-		notebook->priv->motion_notify_handler_id = 0;
-	}       
-}
-
-/* Callbacks */
-static gboolean
-button_release_cb (GulNotebook *notebook, GdkEventButton *event,
-		   gpointer data)
-{
-	if (notebook->priv->drag_in_progress) 
-	{
-		/* ungrab the pointer if it's grabbed */
-		if (gdk_pointer_is_grabbed ())
-		{
-			gdk_pointer_ungrab (GDK_CURRENT_TIME);
-			gtk_grab_remove (GTK_WIDGET(notebook));
-		}
-	}
-
-	/* This must be called even if a drag isn't happening */
-	drag_stop(notebook);
-	return FALSE;
-}
-
-static gboolean 
-motion_notify_cb (GulNotebook *notebook, GdkEventMotion *event, 
-		  gpointer data)
-{
-	int page_num;
-	int result;
-	int abs_x;
-	int abs_y;
-
-	abs_x = (int)event->x_root;
-	abs_y = (int)event->y_root;
-	
-	if (notebook->priv->drag_in_progress == FALSE) 
-	{
-		gint cur_page;
-		
-		cur_page = gtk_notebook_get_current_page (GTK_NOTEBOOK(notebook));
-		drag_start (notebook, notebook, cur_page); 
-	}
-
-	result = is_notebook_at_pointer (notebook, abs_x, abs_y);
-	if (result) 
-	{
-		page_num = find_tab_num_at_pos (notebook, abs_x, abs_y);
-		g_assert (page_num >= -1);
-		move_tab (notebook, page_num);
-	} 
-
-	return FALSE;
-}
-
-static gboolean 
-button_press_cb (GulNotebook *notebook, 
-		 GdkEventButton *event,
-		 gpointer data)
-{
-	gint tab_clicked = find_tab_num_at_pos(notebook, 
-					       event->x_root, 
-					       event->y_root);
-	
-	if (notebook->priv->drag_in_progress)
-	{
-		return TRUE;
-	}
-
-	if ((event->button == 1) && (event->type == GDK_BUTTON_PRESS)
-	    && (tab_clicked != -1))
-	{
-		notebook->priv->motion_notify_handler_id = 
-			g_signal_connect (G_OBJECT (notebook), 
-					  "motion-notify-event",
-					  G_CALLBACK (motion_notify_cb), NULL);
-	}
-
-	return FALSE;
-}
-
-GtkWidget *
-gul_notebook_new (void)
-{
-	return GTK_WIDGET (g_object_new (GUL_NOTEBOOK_TYPE, NULL));
-}
-
 static void
-load_colours (GulNotebook *nb, GtkWidget *label)
-{
-	GtkStyle  *rcstyle;
-	char *tmp;
-	
-	rcstyle = gtk_rc_get_style (label);
-
-        /* this is needed for some (broken?) themes */
-        if (rcstyle == NULL)
-        {
-                rcstyle = gtk_style_new ();
-        }
-        else
-        {
-                g_object_ref (rcstyle);
-        }
-
-	/* make styles */
-        nb->priv->loading_text_style = gtk_style_copy (rcstyle);
-        nb->priv->new_text_style = gtk_style_copy (rcstyle);
-	
-	tmp = eel_gconf_get_string (CONF_TABS_TABBED_LOADING_COLOR);
-        gdk_color_parse (tmp, &(nb->priv->loading_text_style->fg[0]));
-        gdk_color_parse (tmp, &(nb->priv->loading_text_style->fg[2]));
-       	g_free (tmp);
-	
-	tmp = eel_gconf_get_string (CONF_TABS_TABBED_NEW_COLOR);
-        gdk_color_parse (tmp, &(nb->priv->new_text_style->fg[0]));
-        gdk_color_parse (tmp, &(nb->priv->new_text_style->fg[2]));
-        g_free (tmp);
-}
-
-static void
-free_colours (GulNotebook *nb)
-{
-	g_object_unref (nb->priv->loading_text_style);
-	g_object_unref (nb->priv->new_text_style);
-}
-
-static GList *
-tab_remove_from_list (GList *list, GtkWidget *child)
-{
-	GList *rl;
-	GList *l = list;
-	
-	rl = g_list_find (l, child);
-        if (rl)
-	{
-		l = g_list_remove (l, rl->data);
-	}
-
-	return l;
-}
-
-static void
-gul_notebook_switch_page_cb (GtkNotebook *notebook,
-                             GtkNotebookPage *page,
-                             guint page_num,
-                             gpointer data)
-{
-	GulNotebook *nb = GUL_NOTEBOOK (notebook);
-	GtkWidget *child;
-	
-	child = gtk_notebook_get_nth_page (notebook, page_num);
-
-	/* Remove the old page, we dont want to grow unnecessarily
-	 * the list */
-	if (nb->priv->focused_pages)
-	{
-		nb->priv->focused_pages = tab_remove_from_list 
-			(nb->priv->focused_pages, child);
-	}
-
-	nb->priv->focused_pages = g_list_append (nb->priv->focused_pages,
-						 child);
-}
-
-static void
-gul_notebook_init (GulNotebook *notebook)
-{
-	notebook->priv = g_new(GulNotebookPrivate, 1);
-
-	notebook->priv->drag_in_progress = FALSE;
-	notebook->priv->motion_notify_handler_id = 0;
-	notebook->priv->src_notebook = NULL;
-	notebook->priv->src_page = -1;
-	notebook->priv->colors_loaded = FALSE;
-	notebook->priv->focused_pages = NULL;
-	
-	g_signal_connect (notebook, "button-press-event", 
-			  (GCallback)button_press_cb, NULL);
-	g_signal_connect (notebook, "button-release-event", 
-			  (GCallback)button_release_cb, NULL);
-	gtk_widget_add_events (GTK_WIDGET(notebook), GDK_BUTTON1_MOTION_MASK);
-
-	g_signal_connect_after (G_OBJECT (notebook), "switch_page",
-                                G_CALLBACK (gul_notebook_switch_page_cb),
-                                NULL);
-}
-
-static void 
-gul_notebook_finalize (GObject *object)
-{
-	GulNotebook *notebook = GUL_NOTEBOOK(object);
-
-	if (notebook->priv->colors_loaded)
-	{
-		free_colours (notebook);
-	}
-
-	if (notebook->priv->focused_pages)
-	{
-		g_list_free (notebook->priv->focused_pages);
-	}
-	
-	g_free (notebook->priv);
-}
-
-static void
-tab_label_set_size (GtkWidget *notebook, GtkWidget *label)
+tab_label_set_size (GtkWidget *window, GtkWidget *label)
 {
 	int label_width;
-		
-	label_width = notebook->allocation.width/TAB_MAX_SIZE;
+	
+	label_width = window->allocation.width/TAB_NB_MAX;
 
 	if (label_width < TAB_MIN_SIZE) label_width = TAB_MIN_SIZE;
 	
@@ -479,73 +304,380 @@ tab_get_label (GulNotebook *nb, GtkWidget *child)
 }
 
 static void
-tab_label_size_request_cb (GtkWidget *widget,
+tab_label_size_request_cb (GtkWidget *window,
 			   GtkRequisition *requisition,
 			   GtkWidget *child)
 {
 	GtkWidget *hbox;
+	GtkWidget *nb;
+
+	nb = child->parent;
 	
-	hbox = gtk_notebook_get_tab_label (GTK_NOTEBOOK (widget),
+	hbox = gtk_notebook_get_tab_label (GTK_NOTEBOOK (nb),
 					   child);
-	tab_label_set_size (widget, hbox);
+	tab_label_set_size (window, hbox);
 }
 
+
 void
-gul_notebook_set_page_status (GulNotebook *nb,
-			      GtkWidget *child,
-			      GulNotebookPageLoadStatus status)
+gul_notebook_move_page (GulNotebook *src, GulNotebook *dest, 
+			GtkWidget *src_page,  gint dest_page)
 {
-	GtkWidget *label;
+	GtkWidget *tab_label;
+	GtkWidget *menu_label;
 
-	label = tab_get_label (nb, child);
+	tab_label = gtk_notebook_get_tab_label (GTK_NOTEBOOK (src), src_page);
+	menu_label = gtk_notebook_get_menu_label (GTK_NOTEBOOK (src), src_page);
 
-	switch (status)
+	/* We don't want gtk to destroy tab and src_page behind our back */
+	g_object_ref (G_OBJECT (src_page));
+	g_object_ref (G_OBJECT (tab_label));
+	g_object_ref (G_OBJECT (menu_label));
+	gul_notebook_remove_page (GUL_NOTEBOOK (src), src_page);
+	gul_notebook_insert_page (GUL_NOTEBOOK (dest), src_page,
+				  dest_page, TRUE);
+	gtk_notebook_set_tab_label (GTK_NOTEBOOK (dest), src_page, tab_label);
+	gtk_notebook_set_menu_label (GTK_NOTEBOOK (dest), src_page, menu_label);
+	g_object_unref (G_OBJECT (src_page));
+	g_object_unref (G_OBJECT (tab_label));
+	g_object_unref (G_OBJECT (menu_label));
+}
+
+
+
+static void
+move_tab_to_another_notebook(GulNotebook *src, 
+			     GulNotebook *dest, gint dest_page)
+{
+	GtkWidget *child;
+	gint cur_page;
+
+	/* This is getting tricky, the tab was dragged in a notebook
+	 * in another window of the same app, we move the tab
+	 * to that new notebook, and let this notebook handle the
+	 * drag
+	*/
+	g_assert (dest != NULL);
+	g_assert (dest != src);
+
+	/* Move the widgets (tab label and tab content) to the new 
+	 * notebook
+	 */
+	cur_page = gtk_notebook_get_current_page (GTK_NOTEBOOK (src));
+	child    = gtk_notebook_get_nth_page (GTK_NOTEBOOK (src), cur_page);
+	gul_notebook_move_page (src, dest, child, dest_page);
+
+	/* "Give" drag handling to the new notebook */
+	drag_start (dest, src->priv->src_notebook, src->priv->src_page);
+	drag_stop (src);		
+	gtk_grab_remove (GTK_WIDGET (src));
+	
+	dest->priv->motion_notify_handler_id = 
+		g_signal_connect (G_OBJECT (dest), 
+				  "motion-notify-event",
+				  G_CALLBACK (motion_notify_cb),
+				  NULL);
+}
+
+
+static void 
+move_tab (GulNotebook *notebook, gint dest_page_num)
+{
+	gint cur_page_num;
+
+	cur_page_num = gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook));
+
+	if (dest_page_num != cur_page_num)
 	{
-		case GUL_NOTEBOOK_TAB_LOAD_ACTIVATED:
-       	        	gtk_widget_set_style (label, NULL);
-		break;
-		
-		case GUL_NOTEBOOK_TAB_LOAD_STARTED:
-         	       gtk_widget_set_style 
-			       (label, nb->priv->loading_text_style);
-		break;
-		
-        	case GUL_NOTEBOOK_TAB_LOAD_COMPLETED:
-	                gtk_widget_set_style 
-				(label, nb->priv->new_text_style);
-		break;
+		GtkWidget *cur_page;
+		cur_page = gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook),
+						      cur_page_num);
+		gtk_notebook_reorder_child (GTK_NOTEBOOK (notebook), cur_page,
+					    dest_page_num);
+
+		/* Reset the list of newly opened tabs when moving tabs. */
+		g_list_free (notebook->priv->opened_tabs);
+		notebook->priv->opened_tabs = NULL;
 	}
 }
 
+static void 
+drag_start (GulNotebook *notebook, 
+	    GulNotebook *src_notebook, 
+	    gint src_page)
+{
+	notebook->priv->drag_in_progress = TRUE;
+	notebook->priv->src_notebook = src_notebook;
+	notebook->priv->src_page = src_page;
+
+	/* get a new cursor, if necessary */
+	if (!cursor) cursor = gdk_cursor_new (GDK_FLEUR);
+	
+	/* grab the pointer */
+	gtk_grab_add (GTK_WIDGET (notebook));
+	if (!gdk_pointer_is_grabbed ()) {
+		gdk_pointer_grab (GDK_WINDOW(GTK_WIDGET (notebook)->window), 
+				  FALSE,
+				  GDK_BUTTON1_MOTION_MASK | GDK_BUTTON_RELEASE_MASK,
+				  NULL, cursor, GDK_CURRENT_TIME);
+	}
+}
+
+static void 
+drag_stop (GulNotebook *notebook)
+{
+	notebook->priv->drag_in_progress = FALSE;
+	notebook->priv->src_notebook = NULL;
+	notebook->priv->src_page = -1;
+	if (notebook->priv->motion_notify_handler_id != 0) 
+	{
+		g_signal_handler_disconnect (G_OBJECT (notebook), 
+					     notebook->priv->motion_notify_handler_id);
+		notebook->priv->motion_notify_handler_id = 0;
+	}       
+}
+
+/* Callbacks */
+static gboolean
+button_release_cb (GulNotebook *notebook, GdkEventButton *event,
+		   gpointer data)
+{
+	if (notebook->priv->drag_in_progress) 
+	{
+		gint cur_page_num;
+		GtkWidget *cur_page; 
+
+		cur_page_num = 
+			gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook));
+		cur_page = gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook), 
+						      cur_page_num);
+
+		if (!is_in_notebook_window (notebook, event->x_root, event->y_root)) 
+		{
+			/* Tab was detached */
+			g_signal_emit (G_OBJECT(notebook), 
+				       gul_notebook_signals[TAB_DETACHED], 0,
+				       cur_page_num, (gint)event->x_root, 
+				       (gint)event->y_root);
+		}
+		else
+		{
+			/* Tab was dragged and dropped (but it may have stayed
+			   in the same place) */
+			g_signal_emit (G_OBJECT(notebook), 
+				       gul_notebook_signals[TAB_DROPPED], 0,
+				       cur_page, 
+				       notebook->priv->src_notebook, 
+				       notebook->priv->src_page);
+		}
+
+		/* ungrab the pointer if it's grabbed */
+		if (gdk_pointer_is_grabbed ())
+		{
+			gdk_pointer_ungrab (GDK_CURRENT_TIME);
+			gtk_grab_remove (GTK_WIDGET (notebook));
+		}
+	}
+	/* This must be called even if a drag isn't happening */
+	drag_stop (notebook);
+	return FALSE;
+}
+
+
+static gboolean 
+motion_notify_cb (GulNotebook *notebook, GdkEventMotion *event, 
+		  gpointer data)
+{
+	GulNotebook *dest;
+	gint page_num;
+	gint result;
+
+	/* If the notebook only has one tab, we don't want to do
+	 * anything since galeon can't handle empty notebooks
+	 */
+	if (g_list_length (GTK_NOTEBOOK (notebook)->children) <= 1) {
+		return FALSE;
+	}
+	
+	if ((notebook->priv->drag_in_progress == FALSE) 
+	    && (gtk_drag_check_threshold (GTK_WIDGET (notebook),
+					  notebook->priv->x_start, 
+					  notebook->priv->y_start, 
+					  event->x_root, event->y_root)))
+	{
+		gint cur_page;
+		
+		cur_page = gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook));
+		drag_start (notebook, notebook, cur_page); 
+	}
+
+	result = find_notebook_and_tab_at_pos ((gint)event->x_root, 
+					       (gint)event->y_root,
+					       &dest, &page_num);
+
+	if (result != NOT_IN_APP_WINDOWS) 
+	{
+		if (dest != notebook) 
+		{
+			move_tab_to_another_notebook (notebook, dest, 
+						      page_num);
+		} 
+		else 
+		{
+			g_assert (page_num >= -1);
+			move_tab (notebook, page_num);
+		}
+	}
+
+	return FALSE;
+}
+
+static gboolean 
+button_press_cb (GulNotebook *notebook, 
+		 GdkEventButton *event,
+		 gpointer data)
+{
+	gint tab_clicked = find_tab_num_at_pos (notebook, 
+						event->x_root, 
+						event->y_root);
+	
+	if (notebook->priv->drag_in_progress)
+	{
+		return TRUE;
+	}
+
+	if ((event->button == 1) && (event->type == GDK_BUTTON_PRESS)
+	    && (tab_clicked != -1))
+	{
+		notebook->priv->x_start = event->x_root;
+		notebook->priv->y_start = event->y_root;
+		notebook->priv->motion_notify_handler_id = 
+			g_signal_connect (G_OBJECT (notebook), 
+					  "motion-notify-event",
+					  G_CALLBACK (motion_notify_cb), NULL);
+	}
+
+	return FALSE;
+}
+
+GtkWidget *
+gul_notebook_new (void)
+{
+	return GTK_WIDGET (g_object_new (GUL_NOTEBOOK_TYPE, NULL));
+}
+
 static void
-galeon_tab_close_button_clicked_cb (GtkWidget *widget,
-				    GtkWidget *child)
+gul_notebook_switch_page_cb (GtkNotebook *notebook,
+                             GtkNotebookPage *page,
+                             guint page_num,
+                             gpointer data)
+{
+	GulNotebook *nb = GUL_NOTEBOOK (notebook);
+	GtkWidget *child;
+	
+	child = gtk_notebook_get_nth_page (notebook, page_num);
+
+	/* Remove the old page, we dont want to grow unnecessarily
+	 * the list */
+	if (nb->priv->focused_pages)
+	{
+		nb->priv->focused_pages = 
+			g_list_remove (nb->priv->focused_pages, child);
+	}
+
+	nb->priv->focused_pages = g_list_append (nb->priv->focused_pages,
+						 child);
+
+	/* Reset the list of newly opened tabs when switching tabs. */
+	g_list_free (nb->priv->opened_tabs);
+	nb->priv->opened_tabs = NULL;
+}
+
+static void
+gul_notebook_init (GulNotebook *notebook)
+{
+	notebook->priv = g_new (GulNotebookPrivate, 1);
+
+	notebook->priv->drag_in_progress = FALSE;
+	notebook->priv->motion_notify_handler_id = 0;
+	notebook->priv->src_notebook = NULL;
+	notebook->priv->src_page = -1;
+	notebook->priv->focused_pages = NULL;
+	notebook->priv->opened_tabs = NULL;
+	notebook->priv->policy = GTK_POLICY_AUTOMATIC;
+	
+	notebooks = g_list_append (notebooks, notebook);
+
+	g_signal_connect (notebook, "button-press-event", 
+			  (GCallback)button_press_cb, NULL);
+	g_signal_connect (notebook, "button-release-event", 
+			  (GCallback)button_release_cb, NULL);
+	gtk_widget_add_events (GTK_WIDGET (notebook), GDK_BUTTON1_MOTION_MASK);
+
+	g_signal_connect_after (G_OBJECT (notebook), "switch_page",
+                                G_CALLBACK (gul_notebook_switch_page_cb),
+                                NULL);
+}
+
+static void 
+gul_notebook_finalize (GObject *object)
+{
+	GulNotebook *notebook = GUL_NOTEBOOK (object);
+
+	notebooks = g_list_remove (notebooks, notebook);
+
+	if (notebook->priv->focused_pages)
+	{
+		g_list_free (notebook->priv->focused_pages);
+	}
+	g_list_free (notebook->priv->opened_tabs);
+	
+	g_free (notebook->priv);
+}
+
+static void
+tab_close_button_clicked_cb (GtkWidget *widget,
+			     GtkWidget *child)
 {
 	GulNotebook *notebook;
 
-	notebook = g_object_get_data (G_OBJECT (widget), "notebook");
-	
+	notebook = GUL_NOTEBOOK (gtk_widget_get_parent (child));
 	gul_notebook_remove_page (notebook, child);
 }
 
+static void
+tab_close_button_state_changed_cb (GtkWidget	*widget,
+		                   GtkStateType	 previous_state)
+{
+	if (GTK_WIDGET_STATE(widget) == GTK_STATE_ACTIVE)
+	{
+		gtk_widget_set_state (widget, GTK_STATE_NORMAL);
+	}
+}
+
 static GtkWidget *
 tab_build_label (GulNotebook *nb, GtkWidget *child)
 {
 	GtkWidget *label, *hbox, *close_button, *image;
 	int h, w;
 	GClosure *closure;
+	GtkWidget *window;
+
+	window = gtk_widget_get_toplevel (GTK_WIDGET (nb));
 	
 	gtk_icon_size_lookup (GTK_ICON_SIZE_MENU, &w, &h);
 	
-	hbox = gtk_hbox_new (FALSE, 0);
+	/* set hbox spacing and label padding (see below) so that there's an
+	 * equal amount of space around the label */
+	hbox = gtk_hbox_new (FALSE, 4);
 
 	/* setup close button */
 	close_button = gtk_button_new ();
-	gtk_button_set_relief (GTK_BUTTON(close_button), 
+	gtk_button_set_relief (GTK_BUTTON (close_button), 
 			       GTK_RELIEF_NONE);
 	image = gtk_image_new_from_stock (GTK_STOCK_CLOSE,
 					  GTK_ICON_SIZE_MENU);
-	gtk_widget_set_size_request (close_button, w, h);
+	gtk_widget_set_size_request (close_button, w+2, h+2);
 	gtk_container_add (GTK_CONTAINER (close_button), 
 			   image);
 		
@@ -555,30 +687,31 @@ tab_build_label (GulNotebook *nb, GtkWidget *child)
         gtk_misc_set_padding (GTK_MISC (label), 4, 0);
 	gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, TRUE, 0);
 
-	tab_label_set_size (GTK_WIDGET (nb), label);
+	tab_label_set_size (GTK_WIDGET (window), hbox);
 		
-	closure = g_cclosure_new (G_CALLBACK(tab_label_size_request_cb), 
+	closure = g_cclosure_new (G_CALLBACK (tab_label_size_request_cb), 
 				  child, NULL);
   	g_object_watch_closure (G_OBJECT (label), closure);
-  	g_signal_connect_closure_by_id (G_OBJECT(nb),
+  	g_signal_connect_closure_by_id (G_OBJECT (window),
 		g_signal_lookup ("size_request", 
-				 G_OBJECT_TYPE (G_OBJECT(nb))), 0,
+				 G_OBJECT_TYPE (G_OBJECT (window))), 0,
                                  closure,
                                  FALSE);
 	
-	if (!nb->priv->colors_loaded)
-	{
-		load_colours (nb, label);
-	}
-	
 	/* setup button */
 	gtk_box_pack_start (GTK_BOX (hbox), close_button,
                             FALSE, FALSE, 0);
 
 	g_signal_connect (G_OBJECT (close_button), "clicked",
-                          G_CALLBACK (galeon_tab_close_button_clicked_cb),
+                          G_CALLBACK (tab_close_button_clicked_cb),
+                          child);
+
+	/* work around a cosmetic bug in gtk where buttons with GTK_RELIEF_NONE
+	 * have annoyingly visible border only when in GTK_STATE_ACTIVE (ie. in
+	 * inactive tabs) */
+	g_signal_connect (G_OBJECT (close_button), "state_changed",
+                          G_CALLBACK (tab_close_button_state_changed_cb),
                           child);
-	g_object_set_data (G_OBJECT (close_button), "notebook", nb);
 		
 	gtk_widget_show (hbox);
 	gtk_widget_show (label);
@@ -605,7 +738,7 @@ update_tabs_visibility (GulNotebook *nb, gboolean before_inserting)
 	if (before_inserting) tabs_num--;
 	
 	show_tabs = gtk_notebook_get_nth_page (GTK_NOTEBOOK (nb), tabs_num)
-		|| eel_gconf_get_boolean (CONF_TABS_TABBED_ALWAYS_SHOW);
+		|| nb->priv->policy == GTK_POLICY_ALWAYS;
 	
 	gtk_notebook_set_show_tabs (GTK_NOTEBOOK (nb), show_tabs);
 }
@@ -617,14 +750,41 @@ gul_notebook_insert_page (GulNotebook *nb,
 			  gboolean jump_to)
 {
 	GtkWidget *tab_hbox;
+	GtkWidget *label;
 	
 	tab_hbox = tab_build_label (nb, child);
 		
 	update_tabs_visibility (nb, TRUE);
+
+	if (position == GUL_NOTEBOOK_INSERT_GROUPED)
+	{
+		/* Keep a list of newly opened tabs, if the list is empty open the new
+		 * tab after the current one. If it's not, add it after the newly
+		 * opened tabs.
+		 */
+		if (nb->priv->opened_tabs != NULL)
+		{
+			GList *last = g_list_last (nb->priv->opened_tabs);
+			GtkWidget *last_tab = last->data;
+			position = gtk_notebook_page_num 
+				    (GTK_NOTEBOOK (nb), last_tab) + 1;
+		}
+		else
+		{
+			position = gtk_notebook_get_current_page 
+				    (GTK_NOTEBOOK (nb)) + 1;
+		}
+		nb->priv->opened_tabs = 
+			g_list_append (nb->priv->opened_tabs, child);
+	}
 	
-	gtk_notebook_insert_page (GTK_NOTEBOOK (nb), 
-				  child,
-				  tab_hbox, position);
+	label = gtk_label_new (_("Untitled"));
+	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
+	gtk_notebook_insert_page_menu (GTK_NOTEBOOK (nb), 
+				       child,
+				       tab_hbox, 
+				       label, 
+				       position);
 
 	if (jump_to)
 	{
@@ -681,8 +841,10 @@ gul_notebook_remove_page (GulNotebook *nb,
 	}
 	
 	/* Remove the page from the focused pages list */
-	nb->priv->focused_pages = tab_remove_from_list 
-		(nb->priv->focused_pages, child);
+	nb->priv->focused_pages =  g_list_remove (nb->priv->focused_pages, 
+						  child);
+	nb->priv->opened_tabs = g_list_remove (nb->priv->opened_tabs, child);
+		
 		
 	position = gtk_notebook_page_num (GTK_NOTEBOOK (nb), 
 					  child);
@@ -698,6 +860,22 @@ gul_notebook_remove_page (GulNotebook *nb,
 	update_tabs_visibility (nb, FALSE);
 }
 
+void
+gul_notebook_set_page_color (GulNotebook *nb,
+			     GtkWidget *child,
+			     GdkColor *color)
+{
+	GtkWidget *label;
+
+	label = tab_get_label (nb, child);
+	gtk_widget_modify_fg (label, GTK_STATE_NORMAL, color);
+	gtk_widget_modify_fg (label, GTK_STATE_ACTIVE, color);
+
+	label = gtk_notebook_get_menu_label (GTK_NOTEBOOK (nb), child);
+	gtk_widget_modify_fg (label, GTK_STATE_NORMAL, color);
+	gtk_widget_modify_fg (label, GTK_STATE_ACTIVE, color);
+}
+
 void
 gul_notebook_set_page_title (GulNotebook *nb,
 			     GtkWidget *child,
@@ -707,4 +885,22 @@ gul_notebook_set_page_title (GulNotebook *nb,
 	
 	label = tab_get_label (nb, child);
 	gtk_label_set_label (GTK_LABEL (label), title);
+	gtk_notebook_set_menu_label_text (GTK_NOTEBOOK (nb), child, title);
+}
+
+void
+gul_notebook_set_policy (GulNotebook *nb,
+		         GtkPolicyType policy)
+{
+	if (policy == GTK_POLICY_NEVER)
+	{
+		/* FIXME this is because of the HACK in update_tabs_visibility,
+		 * should really get it fixed (or verify it's fixed) instead of
+		 * working around it, but I don't the details how it goes crazy
+		 */
+		g_warning ("GTK_POLICY_NEVER not implemented");
+		return;
+	}
+	nb->priv->policy = policy;
+	update_tabs_visibility (nb, FALSE);
 }
