@@ -370,86 +370,6 @@ gul_string_time_to_string (GTime t)
 	return g_strdup (str);
 }
 
-/**
- * gul_string_simplify_uri: Simplify a local filesystem uri, eg
- * /home/jorn/blah/../txt to /home/jorn/txt. Caller should free string.
- */
-gchar *
-gul_string_simplify_uri (const gchar *uri)
-{
-	gchar **chars, *ret;
-	GString *gstring;
-	gint i;
-	GList *l, *list = NULL;
-	gboolean leading_slash = FALSE;
-
-	if (uri[0] == '/') leading_slash = TRUE;
-	
-	chars = g_strsplit (uri, "/", 0);
-
-	for (i = 0; chars[i] != NULL; i++)
-	{
-		if (strcmp (chars[i], ".") == 0)
-		{
-			g_free (chars[i]);
-			chars[i] = g_strdup ("");
-		}
-		else if (strcmp (chars[i], "..") == 0)
-		{
-		 	GList *l;
-			gint c;
-
-			for (c = i - 1; c > 0 && chars[c] != NULL; c--)
-			{
-				if (strcmp (chars[c], "..") != 0 &&
-				    strlen (chars[c]) > 0)
-				{
-					break;
-				}
-			}
-
-			g_assert (chars[c] != NULL);
-
-			l = g_list_find (list, chars[c]);
-			if (l)
-			{
-				list = g_list_remove (list, l->data);
-				g_free (chars[c]);
-				chars[c] = g_strdup ("");
-			}
-		}
-		else
-		{
-			list = g_list_append (list, chars[i]);
-		}
-	}
-
-	if (leading_slash)
-		gstring = g_string_new ("/");
-	else
-		gstring = g_string_new ("");
-
-	for (l = list; l; l = g_list_next (l))
-	{
-		if (strcmp ((gchar *) l->data, "") == 0) continue;
-
-		if (strcmp (gstring->str,  "") != 0 &&
-		    strcmp (gstring->str, "/") != 0)
-		{
-			g_string_append_c (gstring, '/');
-		}
-		g_string_append (gstring, (gchar *) l->data);
-	}
-
-	ret = gstring->str;
-	
-	g_string_free (gstring, FALSE);
-	g_list_free (list);
-	g_strfreev (chars);
-	  
-	return ret;
-}
-
 /**
  * gul_string_strip_newline: strip newline from an utf8 string
  */
@@ -799,3 +719,11 @@ gul_strsplit_multiple_delimiters_with_quotes (const gchar *string,
 	
 	return str_array;
 }
+
+gboolean 
+gul_string_is_prefix (const gchar *prefix, const gchar *s)
+{
+	size_t ps = strlen (prefix);
+	return !strncmp (prefix, s, ps);
+}
+
